@@ -6,25 +6,23 @@
 // spell-checker:ignore (ToDO) COMFOLLOW Chowner RFILE RFILE's derefer dgid nonblank nonprint nonprinting
 
 use uucore::display::Quotable;
-pub use uucore::entries;
+use uucore::entries;
 use uucore::error::{FromIo, UResult, USimpleError};
-use uucore::perms::{chown_base, options, GidUidOwnerFilter, IfFrom};
-use uucore::{format_usage, help_about, help_usage};
+use uucore::format_usage;
+use uucore::perms::{GidUidOwnerFilter, IfFrom, chown_base, options};
+use uucore::translate;
 
-use clap::{crate_version, Arg, ArgAction, ArgMatches, Command};
+use clap::{Arg, ArgAction, ArgMatches, Command};
 
 use std::fs;
 use std::os::unix::fs::MetadataExt;
 
-const ABOUT: &str = help_about!("chgrp.md");
-const USAGE: &str = help_usage!("chgrp.md");
-
 fn parse_gid_from_str(group: &str) -> Result<u32, String> {
     if let Some(gid_str) = group.strip_prefix(':') {
         // Handle :gid format
         gid_str
             .parse::<u32>()
-            .map_err(|_| format!("invalid group id: '{}'", gid_str))
+            .map_err(|_| translate!("chgrp-error-invalid-group-id", "gid_str" => gid_str))
     } else {
         // Try as group name first
         match entries::grp2gid(group) {
@@ -32,21 +30,24 @@ fn parse_gid_from_str(group: &str) -> Result<u32, String> {
             // If group name lookup fails, try parsing as raw number
             Err(_) => group
                 .parse::<u32>()
-                .map_err(|_| format!("invalid group: '{}'", group)),
+                .map_err(|_| translate!("chgrp-error-invalid-group", "group" => group)),
         }
     }
 }
 
 fn get_dest_gid(matches: &ArgMatches) -> UResult<(Option<u32>, String)> {
     let mut raw_group = String::new();
-    let dest_gid = if let Some(file) = matches.get_one::<String>(options::REFERENCE) {
-        fs::metadata(file)
+    let dest_gid = if let Some(file) = matches.get_one::<std::ffi::OsString>(options::REFERENCE) {
+        let path = std::path::Path::new(file);
+        fs::metadata(path)
             .map(|meta| {
                 let gid = meta.gid();
                 raw_group = entries::gid2grp(gid).unwrap_or_else(|_| gid.to_string());
                 Some(gid)
             })
-            .map_err_context(|| format!("failed to get attributes of {}", file.quote()))?
+            .map_err_context(
+                || translate!("chgrp-error-failed-to-get-attributes", "file" => path.quote()),
+            )?
     } else {
         let group = matches
             .get_one::<String>(options::ARG_GROUP)
@@ -75,8 +76,8 @@ fn parse_gid_and_uid(matches: &ArgMatches) -> UResult<GidUidOwnerFilter> {
             Err(_) => {
                 return Err(USimpleError::new(
                     1,
-                    format!("invalid user: '{}'", from_group),
-                ))
+                    translate!("chgrp-error-invalid-user", "from_group" => from_group),
+                ));
             }
         }
     } else {
@@ -97,23 +98,24 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 }
 
 pub fn uu_app() -> Command {
-    Command::new(uucore::util_name())
-        .version(crate_version!())
-        .about(ABOUT)
-        .override_usage(format_usage(USAGE))
-        .infer_long_args(true)
+    let cmd = Command::new(uucore::util_name())
+        .version(uucore::crate_version!())
+        .about(translate!("chgrp-about"))
+        .override_usage(format_usage(&translate!("chgrp-usage")))
+        .infer_long_args(true);
+    uucore::clap_localization::configure_localized_command(cmd)
         .disable_help_flag(true)
         .arg(
             Arg::new(options::HELP)
                 .long(options::HELP)
-                .help("Print help information.")
+                .help(translate!("chgrp-help-print-help"))
                 .action(ArgAction::Help),
         )
         .arg(
             Arg::new(options::verbosity::CHANGES)
                 .short('c')
                 .long(options::verbosity::CHANGES)
-                .help("like verbose but report only when a change is made")
+                .help(translate!("chgrp-help-changes"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -125,26 +127,26 @@ pub fn uu_app() -> Command {
         .arg(
             Arg::new(options::verbosity::QUIET)
                 .long(options::verbosity::QUIET)
-                .help("suppress most error messages")
+                .help(translate!("chgrp-help-quiet"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::verbosity::VERBOSE)
                 .short('v')
                 .long(options::verbosity::VERBOSE)
-                .help("output a diagnostic for every file processed")
+                .help(translate!("chgrp-help-verbose"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::preserve_root::PRESERVE)
                 .long(options::preserve_root::PRESERVE)
-                .help("fail to operate recursively on '/'")
+                .help(translate!("chgrp-help-preserve-root"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::preserve_root::NO_PRESERVE)
                 .long(options::preserve_root::NO_PRESERVE)
-                .help("do not treat '/' specially (the default)")
+                .help(translate!("chgrp-help-no-preserve-root"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -152,19 +154,20 @@ pub fn uu_app() -> Command {
                 .long(options::REFERENCE)
                 .value_name("RFILE")
                 .value_hint(clap::ValueHint::FilePath)
-                .help("use RFILE's group rather than specifying GROUP values"),
+                .value_parser(clap::value_parser!(std::ffi::OsString))
+                .help(translate!("chgrp-help-reference")),
         )
         .arg(
             Arg::new(options::FROM)
                 .long(options::FROM)
                 .value_name("GROUP")
-                .help("change the group only if its current group matches GROUP"),
+                .help(translate!("chgrp-help-from")),
         )
         .arg(
             Arg::new(options::RECURSIVE)
                 .short('R')
                 .long(options::RECURSIVE)
-                .help("operate on files and directories recursively")
+                .help(translate!("chgrp-help-recursive"))
                 .action(ArgAction::SetTrue),
         )
         // Add common arguments with chgrp, chown & chmod
