@@ -5,78 +5,73 @@
 // spell-checker:ignore NEWROOT Userspec userspec
 //! Errors returned by chroot.
 use std::io::Error;
+use std::path::PathBuf;
 use thiserror::Error;
 use uucore::display::Quotable;
 use uucore::error::UError;
 use uucore::libc;
+use uucore::translate;
 
 /// Errors that can happen while executing chroot.
 #[derive(Debug, Error)]
 pub enum ChrootError {
     /// Failed to enter the specified directory.
-    #[error("cannot chroot to {dir}: {err}", dir = .0.quote(), err = .1)]
-    CannotEnter(String, #[source] Error),
+    #[error("{}", translate!("chroot-error-cannot-enter", "dir" => _0.quote(), "err" => _1))]
+    CannotEnter(PathBuf, #[source] Error),
 
     /// Failed to execute the specified command.
-    #[error("failed to run command {cmd}: {err}", cmd = .0.to_string().quote(), err = .1)]
+    #[error("{}", translate!("chroot-error-command-failed", "cmd" => _0.quote(), "err" => _1))]
     CommandFailed(String, #[source] Error),
 
     /// Failed to find the specified command.
-    #[error("failed to run command {cmd}: {err}", cmd = .0.to_string().quote(), err = .1)]
+    #[error("{}", translate!("chroot-error-command-not-found", "cmd" => _0.quote(), "err" => _1))]
     CommandNotFound(String, #[source] Error),
 
-    #[error("--groups parsing failed")]
+    #[error("{}", translate!("chroot-error-groups-parsing-failed"))]
     GroupsParsingFailed,
 
-    #[error("invalid group: {group}", group = .0.quote())]
+    #[error("{}", translate!("chroot-error-invalid-group", "group" => _0.quote()))]
     InvalidGroup(String),
 
-    #[error("invalid group list: {list}", list = .0.quote())]
+    #[error("{}", translate!("chroot-error-invalid-group-list", "list" => _0.quote()))]
     InvalidGroupList(String),
 
-    /// The given user and group specification was invalid.
-    #[error("invalid userspec: {spec}", spec = .0.quote())]
-    InvalidUserspec(String),
-
     /// The new root directory was not given.
-    #[error(
-        "Missing operand: NEWROOT\nTry '{0} --help' for more information.",
-        uucore::execution_phrase()
-    )]
+    #[error("{}", translate!("chroot-error-missing-newroot", "util_name" => uucore::execution_phrase()))]
     MissingNewRoot,
 
-    #[error("no group specified for unknown uid: {0}")]
+    #[error("{}", translate!("chroot-error-no-group-specified", "uid" => _0))]
     NoGroupSpecified(libc::uid_t),
 
     /// Failed to find the specified user.
-    #[error("invalid user")]
+    #[error("{}", translate!("chroot-error-no-such-user"))]
     NoSuchUser,
 
     /// Failed to find the specified group.
-    #[error("invalid group")]
+    #[error("{}", translate!("chroot-error-no-such-group"))]
     NoSuchGroup,
 
     /// The given directory does not exist.
-    #[error("cannot change root directory to {dir}: no such directory", dir = .0.quote())]
-    NoSuchDirectory(String),
+    #[error("{}", translate!("chroot-error-no-such-directory", "dir" => _0.quote()))]
+    NoSuchDirectory(PathBuf),
 
     /// The call to `setgid()` failed.
-    #[error("cannot set gid to {gid}: {err}", gid = .0, err = .1)]
+    #[error("{}", translate!("chroot-error-set-gid-failed", "gid" => _0, "err" => _1))]
     SetGidFailed(String, #[source] Error),
 
     /// The call to `setgroups()` failed.
-    #[error("cannot set groups: {0}")]
+    #[error("{}", translate!("chroot-error-set-groups-failed", "err" => _0))]
     SetGroupsFailed(Error),
 
     /// The call to `setuid()` failed.
-    #[error("cannot set user to {user}: {err}", user = .0.maybe_quote(), err = .1)]
+    #[error("{}", translate!("chroot-error-set-user-failed", "user" => _0.maybe_quote(), "err" => _1))]
     SetUserFailed(String, #[source] Error),
 }
 
 impl UError for ChrootError {
-    // 125 if chroot itself fails
-    // 126 if command is found but cannot be invoked
-    // 127 if command cannot be found
+    /// 125 if chroot itself fails
+    /// 126 if command is found but cannot be invoked
+    /// 127 if command cannot be found
     fn code(&self) -> i32 {
         match self {
             Self::CommandFailed(_, _) => 126,
