@@ -5,7 +5,7 @@
 
 // spell-checker:ignore (ToDO) INFTY MULT PSKIP accum aftertab beforetab breakwords fmt's formatline linebreak linebreaking linebreaks linelen maxlength minlength nchars noformat noformatline ostream overlen parasplit plass pmatch poffset posn powf prefixindent punct signum slen sstart tabwidth tlen underlen winfo wlen wordlen wordsplits xanti xprefix
 
-use std::io::{BufRead, Lines};
+use std::io::BufRead;
 use std::iter::Peekable;
 use std::slice::Iter;
 use unicode_width::UnicodeWidthChar;
@@ -26,16 +26,112 @@ fn char_width(c: char) -> usize {
     }
 }
 
+/// Return the UTF-8 sequence length implied by a leading byte, or `None` if invalid.
+fn utf8_char_width(byte: u8) -> Option<usize> {
+    // UTF-8 leading-byte ranges per Unicode Standard, Ch. 3, Table 3-7 and RFC 3629.
+    // 00..7F => 1 byte; C2..DF => 2 bytes; E0..EF => 3 bytes; F0..F4 => 4 bytes.
+    // Disallowed bytes include C0..C1 and F5..FF.
+    const ASCII_MAX: u8 = 0x7F;
+    const TWO_BYTE_START: u8 = 0xC2;
+    const TWO_BYTE_END: u8 = 0xDF;
+    const THREE_BYTE_START: u8 = 0xE0;
+    const THREE_BYTE_END: u8 = 0xEF;
+    const FOUR_BYTE_START: u8 = 0xF0;
+    const FOUR_BYTE_END: u8 = 0xF4; // up to U+10FFFF
+
+    if byte <= ASCII_MAX {
+        return Some(1);
+    }
+    if (TWO_BYTE_START..=TWO_BYTE_END).contains(&byte) {
+        return Some(2);
+    }
+    if (THREE_BYTE_START..=THREE_BYTE_END).contains(&byte) {
+        return Some(3);
+    }
+    if (FOUR_BYTE_START..=FOUR_BYTE_END).contains(&byte) {
+        return Some(4);
+    }
+    None
+}
+
+/// Decode a UTF-8 character starting at `start`, returning the char and bytes consumed.
+fn decode_char(bytes: &[u8], start: usize) -> (Option<char>, usize) {
+    let Some(&first) = bytes.get(start) else {
+        return (None, 1);
+    };
+    if first < 0x80 {
+        return (Some(first as char), 1);
+    }
+
+    let Some(width) = utf8_char_width(first) else {
+        return (None, 1);
+    };
+
+    if start + width > bytes.len() {
+        return (None, 1);
+    }
+
+    match std::str::from_utf8(&bytes[start..start + width]) {
+        Ok(s) => (s.chars().next(), width),
+        Err(_) => (None, 1),
+    }
+}
+
+struct DecodedCharInfo {
+    ch: Option<char>,
+    consumed: usize,
+    width: usize,
+    is_ascii: bool,
+}
+
+fn decode_char_info(bytes: &[u8], start: usize) -> DecodedCharInfo {
+    let (ch, consumed) = decode_char(bytes, start);
+    let (width, is_ascii) = match ch {
+        Some(c) => (char_width(c), c.is_ascii()),
+        None => (1, false),
+    };
+    DecodedCharInfo {
+        ch,
+        consumed,
+        width,
+        is_ascii,
+    }
+}
+
+/// Compute display width for a UTF-8 byte slice, treating invalid bytes as width 1.
+fn byte_display_width(bytes: &[u8]) -> usize {
+    let mut width = 0;
+    let mut idx = 0;
+    while idx < bytes.len() {
+        let info = decode_char_info(bytes, idx);
+        width += info.width;
+        idx += info.consumed;
+    }
+    width
+}
+
+/// GNU fmt has a more restrictive definition of whitespace than Unicode.
+/// It only considers ASCII whitespace characters (space, tab, newline, etc.)
+/// and excludes many Unicode whitespace characters like non-breaking spaces.
+fn is_fmt_whitespace(c: char) -> bool {
+    // Only ASCII whitespace characters are considered whitespace in GNU fmt
+    matches!(c, ' ' | '\t' | '\n' | '\r' | '\x0B' | '\x0C')
+}
+
+fn is_fmt_whitespace_byte(b: u8) -> bool {
+    matches!(b, b' ' | b'\t' | b'\n' | b'\r' | 0x0B | 0x0C)
+}
+
 // lines with PSKIP, lacking PREFIX, or which are entirely blank are
 // NoFormatLines; otherwise, they are FormatLines
 #[derive(Debug)]
 pub enum Line {
     FormatLine(FileLine),
-    NoFormatLine(String, bool),
+    NoFormatLine(Vec<u8>, bool),
 }
 
 impl Line {
-    // when we know that it's a FormatLine, as in the ParagraphStream iterator
+    /// when we know that it's a [`Line::FormatLine`], as in the [`ParagraphStream`] iterator
     fn get_formatline(self) -> FileLine {
         match self {
             Self::FormatLine(fl) => fl,
@@ -43,8 +139,8 @@ impl Line {
         }
     }
 
-    // when we know that it's a NoFormatLine, as in the ParagraphStream iterator
-    fn get_noformatline(self) -> (String, bool) {
+    /// when we know that it's a [`Line::NoFormatLine`], as in the [`ParagraphStream`] iterator
+    fn get_noformatline(self) -> (Vec<u8>, bool) {
         match self {
             Self::NoFormatLine(s, b) => (s, b),
             Self::FormatLine(..) => panic!("Found FormatLine when expecting NoFormatLine"),
@@ -56,7 +152,7 @@ impl Line {
 /// the next line or not
 #[derive(Debug)]
 pub struct FileLine {
-    line: String,
+    line: Vec<u8>,
     /// The end of the indent, always the start of the text
     indent_end: usize,
     /// The end of the PREFIX's indent, that is, the spaces before the prefix
@@ -70,75 +166,85 @@ pub struct FileLine {
 /// Iterator that produces a stream of Lines from a file
 pub struct FileLines<'a> {
     opts: &'a FmtOptions,
-    lines: Lines<&'a mut FileOrStdReader>,
+    reader: &'a mut FileOrStdReader,
 }
 
 impl FileLines<'_> {
-    fn new<'b>(opts: &'b FmtOptions, lines: Lines<&'b mut FileOrStdReader>) -> FileLines<'b> {
-        FileLines { opts, lines }
+    fn new<'b>(opts: &'b FmtOptions, reader: &'b mut FileOrStdReader) -> FileLines<'b> {
+        FileLines { opts, reader }
     }
 
     /// returns true if this line should be formatted
-    fn match_prefix(&self, line: &str) -> (bool, usize) {
+    fn match_prefix(&self, line: &[u8]) -> (bool, usize) {
         let Some(prefix) = &self.opts.prefix else {
             return (true, 0);
         };
 
-        FileLines::match_prefix_generic(prefix, line, self.opts.xprefix)
+        FileLines::match_prefix_generic(prefix.as_bytes(), line, self.opts.xprefix)
     }
 
     /// returns true if this line should be formatted
-    fn match_anti_prefix(&self, line: &str) -> bool {
+    fn match_anti_prefix(&self, line: &[u8]) -> bool {
         let Some(anti_prefix) = &self.opts.anti_prefix else {
             return true;
         };
 
-        match FileLines::match_prefix_generic(anti_prefix, line, self.opts.xanti_prefix) {
+        match FileLines::match_prefix_generic(anti_prefix.as_bytes(), line, self.opts.xanti_prefix)
+        {
             (true, _) => false,
             (_, _) => true,
         }
     }
 
-    fn match_prefix_generic(pfx: &str, line: &str, exact: bool) -> (bool, usize) {
+    fn match_prefix_generic(pfx: &[u8], line: &[u8], exact: bool) -> (bool, usize) {
         if line.starts_with(pfx) {
             return (true, 0);
         }
 
         if !exact {
-            // we do it this way rather than byte indexing to support unicode whitespace chars
-            for (i, char) in line.char_indices() {
+            let mut i = 0;
+            while i < line.len() {
                 if line[i..].starts_with(pfx) {
                     return (true, i);
-                } else if !char.is_whitespace() {
+                } else if !is_fmt_whitespace_byte(line[i]) {
                     break;
                 }
+                i += 1;
             }
         }
 
         (false, 0)
     }
 
-    fn compute_indent(&self, string: &str, prefix_end: usize) -> (usize, usize, usize) {
+    fn compute_indent(&self, bytes: &[u8], prefix_end: usize) -> (usize, usize, usize) {
         let mut prefix_len = 0;
         let mut indent_len = 0;
-        let mut indent_end = 0;
-        for (os, c) in string.char_indices() {
-            if os == prefix_end {
+        let mut indent_end = bytes.len();
+        let mut idx = 0;
+        while idx < bytes.len() {
+            if idx == prefix_end {
                 // we found the end of the prefix, so this is the printed length of the prefix here
                 prefix_len = indent_len;
             }
 
-            if (os >= prefix_end) && !c.is_whitespace() {
-                // found first non-whitespace after prefix, this is indent_end
-                indent_end = os;
+            let byte = bytes[idx];
+            if idx >= prefix_end && !is_fmt_whitespace_byte(byte) {
+                indent_end = idx;
                 break;
-            } else if c == '\t' {
-                // compute tab length
-                indent_len = (indent_len / self.opts.tabwidth + 1) * self.opts.tabwidth;
-            } else {
-                // non-tab character
-                indent_len += char_width(c);
             }
+
+            if byte == b'\t' {
+                indent_len = (indent_len / self.opts.tabwidth + 1) * self.opts.tabwidth;
+                idx += 1;
+                continue;
+            }
+
+            let info = decode_char_info(bytes, idx);
+            indent_len += info.width;
+            idx += info.consumed;
+        }
+        if indent_end == bytes.len() {
+            indent_end = idx;
         }
         (indent_end, prefix_len, indent_len)
     }
@@ -148,14 +254,26 @@ impl Iterator for FileLines<'_> {
     type Item = Line;
 
     fn next(&mut self) -> Option<Line> {
-        let n = self.lines.next()?.ok()?;
+        let mut buf = Vec::new();
+        match self.reader.read_until(b'\n', &mut buf) {
+            Ok(0) => return None,
+            Ok(_) => {}
+            Err(_) => return None,
+        }
+        if buf.ends_with(b"\n") {
+            buf.pop();
+            if buf.ends_with(b"\r") {
+                buf.pop();
+            }
+        }
+        let n = buf;
 
         // if this line is entirely whitespace,
         // emit a blank line
         // Err(true) indicates that this was a linebreak,
         // which is important to know when detecting mail headers
-        if n.chars().all(char::is_whitespace) {
-            return Some(Line::NoFormatLine(String::new(), true));
+        if n.iter().all(|&b| is_fmt_whitespace_byte(b)) {
+            return Some(Line::NoFormatLine(Vec::new(), true));
         }
 
         let (pmatch, poffset) = self.match_prefix(&n[..]);
@@ -173,8 +291,8 @@ impl Iterator for FileLines<'_> {
         // following line)
         if pmatch
             && n[poffset + self.opts.prefix.as_ref().map_or(0, |s| s.len())..]
-                .chars()
-                .all(char::is_whitespace)
+                .iter()
+                .all(|&b| is_fmt_whitespace_byte(b))
         {
             return Some(Line::NoFormatLine(n, false));
         }
@@ -199,23 +317,23 @@ impl Iterator for FileLines<'_> {
     }
 }
 
-/// A paragraph : a collection of FileLines that are to be formatted
+/// A paragraph : a collection of [`FileLines`] that are to be formatted
 /// plus info about the paragraph's indentation
 ///
-/// We only retain the String from the FileLine; the other info
+/// We retain the raw bytes from the [`FileLine`]; the other info
 /// is only there to help us in deciding how to merge lines into Paragraphs
 #[derive(Debug)]
 pub struct Paragraph {
     /// the lines of the file
-    lines: Vec<String>,
+    lines: Vec<Vec<u8>>,
     /// string representing the init, that is, the first line's indent
-    pub init_str: String,
+    pub init_str: Vec<u8>,
     /// printable length of the init string considering TABWIDTH
     pub init_len: usize,
-    /// byte location of end of init in first line String
+    /// byte location of end of init in first line buffer
     init_end: usize,
     /// string representing indent
-    pub indent_str: String,
+    pub indent_str: Vec<u8>,
     /// length of above
     pub indent_len: usize,
     /// byte location of end of indent (in crown and tagged mode, only applies to 2nd line and onward)
@@ -234,7 +352,7 @@ pub struct ParagraphStream<'a> {
 
 impl ParagraphStream<'_> {
     pub fn new<'b>(opts: &'b FmtOptions, reader: &'b mut FileOrStdReader) -> ParagraphStream<'b> {
-        let lines = FileLines::new(opts, reader.lines()).peekable();
+        let lines = FileLines::new(opts, reader).peekable();
         // at the beginning of the file, we might find mail headers
         ParagraphStream {
             lines,
@@ -252,10 +370,10 @@ impl ParagraphStream<'_> {
             false
         } else {
             let l_slice = &line.line[..];
-            if l_slice.starts_with("From ") {
+            if l_slice.starts_with(b"From ") {
                 true
             } else {
-                let Some(colon_posn) = l_slice.find(':') else {
+                let Some(colon_posn) = l_slice.iter().position(|&b| b == b':') else {
                     return false;
                 };
 
@@ -265,18 +383,18 @@ impl ParagraphStream<'_> {
                 }
 
                 l_slice[..colon_posn]
-                    .chars()
-                    .all(|x| !matches!(x as usize, y if !(33..=126).contains(&y)))
+                    .iter()
+                    .all(|&b| (33..=126).contains(&(b as usize)) && b != b':')
             }
         }
     }
 }
 
 impl Iterator for ParagraphStream<'_> {
-    type Item = Result<Paragraph, String>;
+    type Item = Result<Paragraph, Vec<u8>>;
 
     #[allow(clippy::cognitive_complexity)]
-    fn next(&mut self) -> Option<Result<Paragraph, String>> {
+    fn next(&mut self) -> Option<Result<Paragraph, Vec<u8>>> {
         // return a NoFormatLine in an Err; it should immediately be output
         let noformat = match self.lines.peek()? {
             Line::FormatLine(_) => false,
@@ -291,10 +409,10 @@ impl Iterator for ParagraphStream<'_> {
         }
 
         // found a FormatLine, now build a paragraph
-        let mut init_str = String::new();
+        let mut init_str = Vec::new();
         let mut init_end = 0;
         let mut init_len = 0;
-        let mut indent_str = String::new();
+        let mut indent_str = Vec::new();
         let mut indent_end = 0;
         let mut indent_len = 0;
         let mut prefix_len = 0;
@@ -318,11 +436,11 @@ impl Iterator for ParagraphStream<'_> {
                     // there can't be any indent or prefixindent because otherwise is_mail_header
                     // would fail since there cannot be any whitespace before the colon in a
                     // valid header field
-                    indent_str.push_str("  ");
+                    indent_str.extend_from_slice(b"  ");
                     indent_len = 2;
                 } else {
                     if self.opts.crown || self.opts.tagged {
-                        init_str.push_str(&fl.line[..fl.indent_end]);
+                        init_str.extend_from_slice(&fl.line[..fl.indent_end]);
                         init_len = fl.indent_len;
                         init_end = fl.indent_end;
                     } else {
@@ -332,7 +450,7 @@ impl Iterator for ParagraphStream<'_> {
                     // these will be overwritten in the 2nd line of crown or tagged mode, but
                     // we are not guaranteed to get to the 2nd line, e.g., if the next line
                     // is a NoFormatLine or None. Thus, we set sane defaults the 1st time around
-                    indent_str.push_str(&fl.line[..fl.indent_end]);
+                    indent_str.extend_from_slice(&fl.line[..fl.indent_end]);
                     indent_len = fl.indent_len;
                     indent_end = fl.indent_end;
 
@@ -346,7 +464,7 @@ impl Iterator for ParagraphStream<'_> {
                     // pretty arbitrary.
                     // Perhaps a better default would be 1 TABWIDTH? But ugh that's so big.
                     if self.opts.tagged {
-                        indent_str.push_str("    ");
+                        indent_str.extend_from_slice(b"    ");
                         indent_len += 4;
                     }
                 }
@@ -373,7 +491,7 @@ impl Iterator for ParagraphStream<'_> {
 
                 // this is part of the same paragraph, get the indent info from this line
                 indent_str.clear();
-                indent_str.push_str(&fl.line[..fl.indent_end]);
+                indent_str.extend_from_slice(&fl.line[..fl.indent_end]);
                 indent_len = fl.indent_len;
                 indent_end = fl.indent_end;
 
@@ -441,11 +559,14 @@ impl<'a> ParaWords<'a> {
                 self.para
                     .lines
                     .iter()
-                    .flat_map(|x| x.split_whitespace())
+                    .flat_map(|x| {
+                        x.split(|b| is_fmt_whitespace_byte(*b))
+                            .filter(|segment| !segment.is_empty())
+                    })
                     .map(|x| WordInfo {
                         word: x,
                         word_start: 0,
-                        word_nchars: x.len(), // OK for mail headers; only ASCII allowed (unicode is escaped)
+                        word_nchars: byte_display_width(x),
                         before_tab: None,
                         after_tab: 0,
                         sentence_start: false,
@@ -484,24 +605,22 @@ impl<'a> ParaWords<'a> {
 
 struct WordSplit<'a> {
     opts: &'a FmtOptions,
-    string: &'a str,
+    bytes: &'a [u8],
     length: usize,
     position: usize,
     prev_punct: bool,
 }
 
 impl WordSplit<'_> {
-    fn analyze_tabs(&self, string: &str) -> (Option<usize>, usize, Option<usize>) {
-        // given a string, determine (length before tab) and (printed length after first tab)
-        // if there are no tabs, beforetab = -1 and aftertab is the printed length
+    fn analyze_tabs(&self, bytes: &[u8]) -> (Option<usize>, usize, Option<usize>) {
         let mut beforetab = None;
         let mut aftertab = 0;
         let mut word_start = None;
-        for (os, c) in string.char_indices() {
-            if !c.is_whitespace() {
-                word_start = Some(os);
+        for (idx, b) in bytes.iter().enumerate() {
+            if !is_fmt_whitespace_byte(*b) {
+                word_start = Some(idx);
                 break;
-            } else if c == '\t' {
+            } else if *b == b'\t' {
                 if beforetab.is_none() {
                     beforetab = Some(aftertab);
                     aftertab = 0;
@@ -514,28 +633,50 @@ impl WordSplit<'_> {
         }
         (beforetab, aftertab, word_start)
     }
-}
 
-impl WordSplit<'_> {
-    fn new<'b>(opts: &'b FmtOptions, string: &'b str) -> WordSplit<'b> {
-        // wordsplits *must* start at a non-whitespace character
-        let trim_string = string.trim_start();
+    fn new<'b>(opts: &'b FmtOptions, bytes: &'b [u8]) -> WordSplit<'b> {
+        let start = bytes
+            .iter()
+            .position(|&b| !is_fmt_whitespace_byte(b))
+            .unwrap_or(bytes.len());
+        let trimmed = &bytes[start..];
         WordSplit {
             opts,
-            string: trim_string,
-            length: string.len(),
+            bytes: trimmed,
+            length: trimmed.len(),
             position: 0,
             prev_punct: false,
         }
     }
 
-    fn is_punctuation(c: char) -> bool {
-        matches!(c, '!' | '.' | '?')
+    fn is_punctuation_byte(b: u8) -> bool {
+        matches!(b, b'!' | b'.' | b'?')
+    }
+
+    fn scan_word_end(&self, word_start: usize) -> (usize, usize, Option<u8>) {
+        let mut word_nchars = 0;
+        let mut idx = word_start;
+        let mut last_ascii = None;
+        while idx < self.length {
+            let info = decode_char_info(self.bytes, idx);
+            let is_whitespace = info.is_ascii && info.ch.is_some_and(is_fmt_whitespace);
+            if is_whitespace {
+                break;
+            }
+            word_nchars += info.width;
+            if info.is_ascii {
+                last_ascii = info.ch.map(|c| c as u8);
+            } else {
+                last_ascii = None;
+            }
+            idx += info.consumed;
+        }
+        (idx, word_nchars, last_ascii)
     }
 }
 
 pub struct WordInfo<'a> {
-    pub word: &'a str,
+    pub word: &'a [u8],
     pub word_start: usize,
     pub word_nchars: usize,
     pub before_tab: Option<usize>,
@@ -559,7 +700,7 @@ impl<'a> Iterator for WordSplit<'a> {
 
         // find the start of the next word, and record if we find a tab character
         let (before_tab, after_tab, word_start) =
-            if let (b, a, Some(s)) = self.analyze_tabs(&self.string[old_position..]) {
+            if let (b, a, Some(s)) = self.analyze_tabs(&self.bytes[old_position..]) {
                 (b, a, s + old_position)
             } else {
                 self.position = self.length;
@@ -569,18 +710,8 @@ impl<'a> Iterator for WordSplit<'a> {
         // find the beginning of the next whitespace
         // note that this preserves the invariant that self.position
         // points to whitespace character OR end of string
-        let mut word_nchars = 0;
-        self.position = match self.string[word_start..].find(|x: char| {
-            if x.is_whitespace() {
-                true
-            } else {
-                word_nchars += char_width(x);
-                false
-            }
-        }) {
-            None => self.length,
-            Some(s) => s + word_start,
-        };
+        let (next_position, word_nchars, last_ascii) = self.scan_word_end(word_start);
+        self.position = next_position;
 
         let word_start_relative = word_start - old_position;
         // if the previous sentence was punctuation and this sentence has >2 whitespace or one tab, is a new sentence.
@@ -588,16 +719,14 @@ impl<'a> Iterator for WordSplit<'a> {
             self.prev_punct && (before_tab.is_some() || word_start_relative > 1);
 
         // now record whether this word ends in punctuation
-        self.prev_punct = match self.string[..self.position].chars().next_back() {
-            Some(ch) => WordSplit::is_punctuation(ch),
-            _ => panic!("fatal: expected word not to be empty"),
-        };
+        let ends_punct = last_ascii.is_some_and(WordSplit::is_punctuation_byte);
+        self.prev_punct = ends_punct;
 
         let (word, word_start_relative, before_tab, after_tab) = if self.opts.uniform {
-            (&self.string[word_start..self.position], 0, None, 0)
+            (&self.bytes[word_start..self.position], 0, None, 0)
         } else {
             (
-                &self.string[old_position..self.position],
+                &self.bytes[old_position..self.position],
                 word_start_relative,
                 before_tab,
                 after_tab,
@@ -611,7 +740,7 @@ impl<'a> Iterator for WordSplit<'a> {
             before_tab,
             after_tab,
             sentence_start: is_start_of_sentence,
-            ends_punct: self.prev_punct,
+            ends_punct,
             new_line,
         })
     }
