@@ -5,25 +5,25 @@
 
 // spell-checker:ignore (ToDOs) corasick memchr Roff trunc oset iset CHARCLASS
 
-use clap::{crate_version, Arg, ArgAction, Command};
-use regex::Regex;
 use std::cmp;
-use std::collections::{BTreeSet, HashMap, HashSet};
-use std::error::Error;
-use std::fmt::{Display, Formatter, Write as FmtWrite};
+use std::cmp::PartialEq;
+use std::collections::{BTreeSet, HashSet};
+use std::ffi::{OsStr, OsString};
+use std::fmt::Write as FmtWrite;
 use std::fs::File;
-use std::io::{stdin, stdout, BufRead, BufReader, BufWriter, Read, Write};
+use std::io::{BufRead, BufReader, BufWriter, Read, Write, stdin, stdout};
 use std::num::ParseIntError;
+use std::path::Path;
+
+use clap::{Arg, ArgAction, Command};
+use regex::Regex;
+use thiserror::Error;
 use uucore::display::Quotable;
-use uucore::error::{FromIo, UError, UResult};
-use uucore::{format_usage, help_about, help_usage};
+use uucore::error::{FromIo, UError, UResult, USimpleError, UUsageError};
+use uucore::format_usage;
+use uucore::translate;
 
-const USAGE: &str = help_usage!("ptx.md");
-const ABOUT: &str = help_about!("ptx.md");
-
-const REGEX_CHARCLASS: &str = "^-]\\";
-
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
 enum OutFormat {
     Dumb,
     Roff,
@@ -43,6 +43,7 @@ struct Config {
     context_regex: String,
     line_width: usize,
     gap_size: usize,
+    sentence_regex: Option<String>,
 }
 
 impl Default for Config {
@@ -59,6 +60,7 @@ impl Default for Config {
             context_regex: "\\w+".to_owned(),
             line_width: 72,
             gap_size: 3,
+            sentence_regex: None,
         }
     }
 }
@@ -68,11 +70,14 @@ fn read_word_filter_file(
     option: &str,
 ) -> std::io::Result<HashSet<String>> {
     let filename = matches
-        .get_one::<String>(option)
-        .expect("parsing options failed!")
-        .to_string();
-    let file = File::open(filename)?;
-    let reader = BufReader::new(file);
+        .get_one::<OsString>(option)
+        .expect("parsing options failed!");
+    let reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
+        Box::new(stdin())
+    } else {
+        let file = File::open(Path::new(filename))?;
+        Box::new(file)
+    });
     let mut words: HashSet<String> = HashSet::new();
     for word in reader.lines() {
         words.insert(word?);
@@ -86,9 +91,14 @@ fn read_char_filter_file(
     option: &str,
 ) -> std::io::Result<HashSet<char>> {
     let filename = matches
-        .get_one::<String>(option)
+        .get_one::<OsString>(option)
         .expect("parsing options failed!");
-    let mut reader = File::open(filename)?;
+    let mut reader: Box<dyn Read> = if filename == "-" {
+        Box::new(stdin())
+    } else {
+        let file = File::open(Path::new(filename))?;
+        Box::new(file)
+    };
     let mut buffer = String::new();
     reader.read_to_string(&mut buffer)?;
     Ok(buffer.chars().collect())
@@ -144,7 +154,7 @@ impl WordFilter {
                     if v.is_empty() {
                         None
                     } else {
-                        Some(v.to_string())
+                        Some(v.to_owned())
                     }
                 }
                 None => None,
@@ -155,18 +165,10 @@ impl WordFilter {
         let reg = match arg_reg {
             Some(arg_reg) => arg_reg,
             None => {
-                if break_set.is_some() {
+                if let Some(break_set) = break_set {
                     format!(
                         "[^{}]+",
-                        break_set
-                            .unwrap()
-                            .into_iter()
-                            .map(|c| if REGEX_CHARCLASS.contains(c) {
-                                format!("\\{c}")
-                            } else {
-                                c.to_string()
-                            })
-                            .collect::<String>()
+                        regex::escape(&break_set.into_iter().collect::<String>())
                     )
                 } else if config.gnu_ext {
                     "\\w+".to_owned()
@@ -192,59 +194,54 @@ struct WordRef {
     local_line_nr: usize,
     position: usize,
     position_end: usize,
-    filename: String,
+    filename: OsString,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Error)]
 enum PtxError {
-    DumbFormat,
-    NotImplemented(&'static str),
+    #[error("{0}")]
     ParseError(ParseIntError),
 }
 
-impl Error for PtxError {}
 impl UError for PtxError {}
 
-impl Display for PtxError {
-    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
-        match self {
-            Self::DumbFormat => {
-                write!(f, "There is no dumb format with GNU extensions disabled")
-            }
-            Self::NotImplemented(s) => write!(f, "{s} not implemented yet"),
-            Self::ParseError(e) => e.fmt(f),
-        }
-    }
-}
-
-fn get_config(matches: &clap::ArgMatches) -> UResult<Config> {
+fn get_config(matches: &mut clap::ArgMatches) -> UResult<Config> {
     let mut config = Config::default();
     let err_msg = "parsing options failed";
     if matches.get_flag(options::TRADITIONAL) {
         config.gnu_ext = false;
         config.format = OutFormat::Roff;
         "[^ \t\n]+".clone_into(&mut config.context_regex);
-    } else {
-        return Err(PtxError::NotImplemented("GNU extensions").into());
     }
-    if matches.contains_id(options::SENTENCE_REGEXP) {
-        return Err(PtxError::NotImplemented("-S").into());
+    if let Some(regex) = matches.remove_one::<String>(options::SENTENCE_REGEXP) {
+        // TODO: The regex crate used here is not fully compatible with GNU's regex implementation.
+        // For example, it does not support backreferences.
+        // In the future, we might want to switch to the onig crate (like expr does) for better compatibility.
+
+        // Verify regex is valid and doesn't match empty string
+        if let Ok(re) = Regex::new(&regex) {
+            if re.is_match("") {
+                return Err(USimpleError::new(1, translate!("ptx-error-empty-regexp")));
+            }
+        }
+
+        config.sentence_regex = Some(regex);
     }
     config.auto_ref = matches.get_flag(options::AUTO_REFERENCE);
     config.input_ref = matches.get_flag(options::REFERENCES);
-    config.right_ref &= matches.get_flag(options::RIGHT_SIDE_REFS);
+    config.right_ref = matches.get_flag(options::RIGHT_SIDE_REFS);
     config.ignore_case = matches.get_flag(options::IGNORE_CASE);
     if matches.contains_id(options::MACRO_NAME) {
-        config.macro_name = matches
+        matches
             .get_one::<String>(options::MACRO_NAME)
             .expect(err_msg)
-            .to_string();
+            .clone_into(&mut config.macro_name);
     }
     if matches.contains_id(options::FLAG_TRUNCATION) {
-        config.trunc_str = matches
+        matches
             .get_one::<String>(options::FLAG_TRUNCATION)
             .expect(err_msg)
-            .to_string();
+            .clone_into(&mut config.trunc_str);
     }
     if matches.contains_id(options::WIDTH) {
         config.line_width = matches
@@ -260,10 +257,17 @@ fn get_config(matches: &clap::ArgMatches) -> UResult<Config> {
             .parse()
             .map_err(PtxError::ParseError)?;
     }
-    if matches.get_flag(options::FORMAT_ROFF) {
+    if let Some(format) = matches.get_one::<String>(options::FORMAT) {
+        config.format = match format.as_str() {
+            "roff" => OutFormat::Roff,
+            "tex" => OutFormat::Tex,
+            _ => unreachable!("should be caught by clap"),
+        };
+    }
+    if matches.get_flag(options::format::ROFF) {
         config.format = OutFormat::Roff;
     }
-    if matches.get_flag(options::FORMAT_TEX) {
+    if matches.get_flag(options::format::TEX) {
         config.format = OutFormat::Tex;
     }
     Ok(config)
@@ -275,51 +279,71 @@ struct FileContent {
     offset: usize,
 }
 
-type FileMap = HashMap<String, FileContent>;
+type FileMap = Vec<(OsString, FileContent)>;
 
-fn read_input(input_files: &[String], config: &Config) -> std::io::Result<FileMap> {
-    let mut file_map: FileMap = HashMap::new();
-    let mut files = Vec::new();
-    if input_files.is_empty() {
-        files.push("-");
-    } else if config.gnu_ext {
-        for file in input_files {
-            files.push(file);
-        }
-    } else {
-        files.push(&input_files[0]);
-    }
+fn read_input(input_files: &[OsString], config: &Config) -> std::io::Result<FileMap> {
+    let mut file_map: FileMap = FileMap::new();
     let mut offset: usize = 0;
-    for filename in files {
-        let reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
+
+    let sentence_splitter = config
+        .sentence_regex
+        .as_ref()
+        .and_then(|re_str| Regex::new(re_str).ok());
+
+    for filename in input_files {
+        let mut reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
             Box::new(stdin())
         } else {
-            let file = File::open(filename)?;
+            let file = File::open(Path::new(filename))?;
             Box::new(file)
         });
-        let lines: Vec<String> = reader.lines().collect::<std::io::Result<Vec<String>>>()?;
+
+        let lines = read_lines(sentence_splitter.as_ref(), &mut reader)?;
 
         // Indexing UTF-8 string requires walking from the beginning, which can hurts performance badly when the line is long.
         // Since we will be jumping around the line a lot, we dump the content into a Vec<char>, which can be indexed in constant time.
         let chars_lines: Vec<Vec<char>> = lines.iter().map(|x| x.chars().collect()).collect();
         let size = lines.len();
-        file_map.insert(
-            filename.to_owned(),
+        file_map.push((
+            filename.clone(),
             FileContent {
                 lines,
                 chars_lines,
                 offset,
             },
-        );
+        ));
         offset += size;
     }
     Ok(file_map)
 }
 
+fn read_lines(
+    sentence_splitter: Option<&Regex>,
+    reader: &mut dyn BufRead,
+) -> std::io::Result<Vec<String>> {
+    if let Some(re) = sentence_splitter {
+        let mut buffer = String::new();
+        reader.read_to_string(&mut buffer)?;
+
+        Ok(re
+            .split(&buffer)
+            .map(|s| s.replace('\n', " ")) // ptx behavior: newlines become spaces inside sentences
+            .filter(|s| !s.is_empty()) // remove empty sentences
+            .collect())
+    } else {
+        reader.lines().collect()
+    }
+}
+
 /// Go through every lines in the input files and record each match occurrence as a `WordRef`.
 fn create_word_set(config: &Config, filter: &WordFilter, file_map: &FileMap) -> BTreeSet<WordRef> {
-    let reg = Regex::new(&filter.word_regex).unwrap();
-    let ref_reg = Regex::new(&config.context_regex).unwrap();
+    let Some(reg) = Regex::new(&filter.word_regex).ok() else {
+        return BTreeSet::new();
+    };
+    let Some(ref_reg) = Regex::new(&config.context_regex).ok() else {
+        return BTreeSet::new();
+    };
+
     let mut word_set: BTreeSet<WordRef> = BTreeSet::new();
     for (file, lines) in file_map {
         let mut count: usize = 0;
@@ -337,14 +361,14 @@ fn create_word_set(config: &Config, filter: &WordFilter, file_map: &FileMap) ->
                     continue;
                 }
                 let mut word = line[beg..end].to_owned();
-                if filter.only_specified && !(filter.only_set.contains(&word)) {
+                if filter.only_specified && !filter.only_set.contains(&word) {
                     continue;
                 }
                 if filter.ignore_specified && filter.ignore_set.contains(&word) {
                     continue;
                 }
                 if config.ignore_case {
-                    word = word.to_lowercase();
+                    word = word.to_uppercase();
                 }
                 word_set.insert(WordRef {
                     word,
@@ -363,11 +387,15 @@ fn create_word_set(config: &Config, filter: &WordFilter, file_map: &FileMap) ->
 
 fn get_reference(config: &Config, word_ref: &WordRef, line: &str, context_reg: &Regex) -> String {
     if config.auto_ref {
-        format!(
-            "{}:{}",
-            word_ref.filename.maybe_quote(),
-            word_ref.local_line_nr + 1
-        )
+        if word_ref.filename == "-" {
+            format!(":{}", word_ref.local_line_nr + 1)
+        } else {
+            format!(
+                "{}:{}",
+                word_ref.filename.maybe_quote(),
+                word_ref.local_line_nr + 1
+            )
+        }
     } else if config.input_ref {
         let (beg, end) = match context_reg.find(line) {
             Some(x) => (x.start(), x.end()),
@@ -415,7 +443,7 @@ fn trim_idx(s: &[char], beg: usize, end: usize) -> (usize, usize) {
     while b < e && s[b].is_whitespace() {
         b += 1;
     }
-    while b < e && s[e - 1].is_whitespace() {
+    while beg < e && s[e - 1].is_whitespace() {
         e -= 1;
     }
     (b, e)
@@ -495,7 +523,18 @@ fn get_output_chunks(
     let tail_end = trim_broken_word_right(all_after, tail_beg, tail_end);
 
     // trim away whitespace again.
-    let (tail_beg, tail_end) = trim_idx(all_after, tail_beg, tail_end);
+    let (tail_beg, mut tail_end) = trim_idx(all_after, tail_beg, tail_end);
+    // Fix: Manually trim trailing char (like "a") that are preceded by a space.
+    // This handles cases like "is a" which are not correctly trimmed by the
+    // preceding functions.
+    if tail_end >= 2
+        && (tail_end - 2) > tail_beg
+        && all_after[tail_end - 2].is_whitespace()
+        && !all_after[tail_end - 1].is_whitespace()
+    {
+        tail_end -= 1;
+        (_, tail_end) = trim_idx(all_after, tail_beg, tail_end);
+    }
 
     // and get the string
     let tail_str: String = all_after[tail_beg..tail_end].iter().collect();
@@ -523,19 +562,21 @@ fn get_output_chunks(
     // and get the string.
     let head_str: String = all_before[head_beg..head_end].iter().collect();
     head.push_str(&head_str);
+    //The TeX mode does not output truncation characters.
+    if config.format != OutFormat::Tex {
+        // put right context truncation string if needed
+        if after_end != all_after.len() && tail_beg == tail_end {
+            after.push_str(&config.trunc_str);
+        } else if after_end != all_after.len() && tail_end != all_after.len() {
+            tail.push_str(&config.trunc_str);
+        }
 
-    // put right context truncation string if needed
-    if after_end != all_after.len() && tail_beg == tail_end {
-        after.push_str(&config.trunc_str);
-    } else if after_end != all_after.len() && tail_end != all_after.len() {
-        tail.push_str(&config.trunc_str);
-    }
-
-    // put left context truncation string if needed
-    if before_beg != 0 && head_beg == head_end {
-        before = format!("{}{}", config.trunc_str, before);
-    } else if before_beg != 0 && head_beg != 0 {
-        head = format!("{}{}", config.trunc_str, head);
+        // put left context truncation string if needed
+        if before_beg != 0 && head_beg == head_end {
+            before = format!("{}{before}", config.trunc_str);
+        } else if before_beg != 0 && head_beg != 0 {
+            head = format!("{}{head}", config.trunc_str);
+        }
     }
 
     (tail, before, after, head)
@@ -565,26 +606,14 @@ fn format_tex_line(
 ) -> String {
     let mut output = String::new();
     write!(output, "\\{} ", config.macro_name).unwrap();
-    let all_before = if config.input_ref {
-        let before = &line[0..word_ref.position];
-        let before_start_trim_offset =
-            word_ref.position - before.trim_start_matches(reference).trim_start().len();
-        let before_end_index = before.len();
-        &chars_line[before_start_trim_offset..cmp::max(before_end_index, before_start_trim_offset)]
-    } else {
-        let before_chars_trim_idx = (0, word_ref.position);
-        &chars_line[before_chars_trim_idx.0..before_chars_trim_idx.1]
-    };
-    let keyword = &line[word_ref.position..word_ref.position_end];
-    let after_chars_trim_idx = (word_ref.position_end, chars_line.len());
-    let all_after = &chars_line[after_chars_trim_idx.0..after_chars_trim_idx.1];
-    let (tail, before, after, head) = get_output_chunks(all_before, keyword, all_after, config);
+    let (tail, before, keyword, after, head) =
+        prepare_line_chunks(config, word_ref, line, chars_line, reference);
     write!(
         output,
         "{{{0}}}{{{1}}}{{{2}}}{{{3}}}{{{4}}}",
         format_tex_field(&tail),
         format_tex_field(&before),
-        format_tex_field(keyword),
+        format_tex_field(&keyword),
         format_tex_field(&after),
         format_tex_field(&head),
     )
@@ -595,6 +624,75 @@ fn format_tex_line(
     output
 }
 
+fn format_dumb_line(
+    config: &Config,
+    word_ref: &WordRef,
+    line: &str,
+    chars_line: &[char],
+    reference: &str,
+) -> String {
+    let (tail, before, keyword, after, head) =
+        prepare_line_chunks(config, word_ref, line, chars_line, reference);
+
+    // Calculate the position for the left part
+    // The left part consists of tail (if present) + space + before
+    let left_part = if tail.is_empty() {
+        before
+    } else if before.is_empty() {
+        tail
+    } else {
+        format!("{tail} {before}")
+    };
+
+    // Calculate the position for the right part
+    let right_part = if head.is_empty() {
+        after
+    } else if after.is_empty() {
+        head
+    } else {
+        format!("{after} {head}")
+    };
+
+    // Calculate the width for the left half (before the keyword)
+    let half_width = cmp::max(config.line_width / 2, config.gap_size);
+
+    let left_part_len = if left_part.contains(&config.trunc_str) {
+        left_part.len() - config.trunc_str.len()
+    } else {
+        left_part.len()
+    };
+
+    // Right-justify the left part within the left half
+    let padding = if left_part.len() < half_width {
+        half_width - left_part_len
+    } else {
+        0
+    };
+
+    // Build the output line with padding, left part, gap, keyword, and right part
+    let mut output = String::new();
+    output.push_str(&" ".repeat(padding));
+    output.push_str(&left_part);
+
+    // Add gap before keyword
+    output.push_str(&" ".repeat(config.gap_size));
+
+    output.push_str(&keyword);
+    output.push_str(&right_part);
+
+    // Add reference if needed
+    if config.auto_ref || config.input_ref {
+        if config.right_ref {
+            output.push(' ');
+            output.push_str(reference);
+        } else {
+            output = format!("{reference} {output}");
+        }
+    }
+
+    output
+}
+
 fn format_roff_field(s: &str) -> String {
     s.replace('\"', "\"\"")
 }
@@ -608,26 +706,14 @@ fn format_roff_line(
 ) -> String {
     let mut output = String::new();
     write!(output, ".{}", config.macro_name).unwrap();
-    let all_before = if config.input_ref {
-        let before = &line[0..word_ref.position];
-        let before_start_trim_offset =
-            word_ref.position - before.trim_start_matches(reference).trim_start().len();
-        let before_end_index = before.len();
-        &chars_line[before_start_trim_offset..cmp::max(before_end_index, before_start_trim_offset)]
-    } else {
-        let before_chars_trim_idx = (0, word_ref.position);
-        &chars_line[before_chars_trim_idx.0..before_chars_trim_idx.1]
-    };
-    let keyword = &line[word_ref.position..word_ref.position_end];
-    let after_chars_trim_idx = (word_ref.position_end, chars_line.len());
-    let all_after = &chars_line[after_chars_trim_idx.0..after_chars_trim_idx.1];
-    let (tail, before, after, head) = get_output_chunks(all_before, keyword, all_after, config);
+    let (tail, before, keyword, after, head) =
+        prepare_line_chunks(config, word_ref, line, chars_line, reference);
     write!(
         output,
         " \"{}\" \"{}\" \"{}{}\" \"{}\"",
         format_roff_field(&tail),
         format_roff_field(&before),
-        format_roff_field(keyword),
+        format_roff_field(&keyword),
         format_roff_field(&after),
         format_roff_field(&head)
     )
@@ -638,24 +724,86 @@ fn format_roff_line(
     output
 }
 
-fn write_traditional_output(
+/// Extract and prepare text chunks for formatting in both TeX and roff output
+fn prepare_line_chunks(
     config: &Config,
+    word_ref: &WordRef,
+    line: &str,
+    chars_line: &[char],
+    reference: &str,
+) -> (String, String, String, String, String) {
+    // Convert byte positions to character positions
+    let ref_char_position = line[..word_ref.position].chars().count();
+    let char_position_end = ref_char_position
+        + line[word_ref.position..word_ref.position_end]
+            .chars()
+            .count();
+
+    // Extract the text before the keyword
+    let all_before = if config.input_ref {
+        let before = &line[..word_ref.position];
+        let before_char_count = before.chars().count();
+        let trimmed_char_count = before
+            .trim_start_matches(reference)
+            .trim_start()
+            .chars()
+            .count();
+        let trim_offset = before_char_count - trimmed_char_count;
+        &chars_line[trim_offset..before_char_count]
+    } else {
+        &chars_line[..ref_char_position]
+    };
+
+    // Extract the keyword and text after it
+    let keyword = line[word_ref.position..word_ref.position_end].to_string();
+    let all_after = &chars_line[char_position_end..];
+
+    // Get formatted output chunks
+    let (tail, before, after, head) = get_output_chunks(all_before, &keyword, all_after, config);
+
+    (tail, before, keyword, after, head)
+}
+
+fn write_traditional_output(
+    config: &mut Config,
     file_map: &FileMap,
     words: &BTreeSet<WordRef>,
-    output_filename: &str,
+    output_filename: &OsStr,
 ) -> UResult<()> {
-    let mut writer: BufWriter<Box<dyn Write>> = BufWriter::new(if output_filename == "-" {
-        Box::new(stdout())
-    } else {
-        let file = File::create(output_filename).map_err_context(String::new)?;
-        Box::new(file)
-    });
+    let mut writer: BufWriter<Box<dyn Write>> =
+        BufWriter::new(if output_filename == OsStr::new("-") {
+            Box::new(stdout())
+        } else {
+            let file = File::create(output_filename)
+                .map_err_context(|| output_filename.quote().to_string())?;
+            Box::new(file)
+        });
 
     let context_reg = Regex::new(&config.context_regex).unwrap();
 
+    if !config.right_ref {
+        let max_ref_len = if config.auto_ref {
+            get_auto_max_reference_len(words)
+        } else {
+            0
+        };
+
+        // Use saturating_sub to prevent panic if the reference is wider than the line width.
+        config.line_width = config.line_width.saturating_sub(max_ref_len);
+    }
+
     for word_ref in words {
-        let file_map_value: &FileContent = file_map
-            .get(&(word_ref.filename))
+        // Since `ptx` accepts duplicate file arguments (e.g., `ptx file file`),
+        // simply looking up by filename is ambiguous.
+        // We use the `global_line_nr` (which is unique across the entire input stream)
+        // to identify which file covers this line.
+        let (_, file_map_value) = file_map
+            .iter()
+            .find(|(name, content)| {
+                name == &word_ref.filename
+                    && word_ref.global_line_nr >= content.offset
+                    && word_ref.global_line_nr < content.offset + content.lines.len()
+            })
             .expect("Missing file in file map");
         let FileContent {
             ref lines,
@@ -683,25 +831,64 @@ fn write_traditional_output(
                 &chars_lines[word_ref.local_line_nr],
                 &reference,
             ),
-            OutFormat::Dumb => {
-                return Err(PtxError::DumbFormat.into());
-            }
+            OutFormat::Dumb => format_dumb_line(
+                config,
+                word_ref,
+                &lines[word_ref.local_line_nr],
+                &chars_lines[word_ref.local_line_nr],
+                &reference,
+            ),
         };
-        writeln!(writer, "{output_line}").map_err_context(String::new)?;
+        writeln!(writer, "{output_line}")
+            .map_err_context(|| translate!("ptx-error-write-failed"))?;
     }
+
+    writer
+        .flush()
+        .map_err_context(|| translate!("ptx-error-write-failed"))?;
+
     Ok(())
 }
 
+fn get_auto_max_reference_len(words: &BTreeSet<WordRef>) -> usize {
+    //Get the maximum length of the reference field
+    let line_num = words
+        .iter()
+        .map(|w| {
+            if w.local_line_nr == 0 {
+                1
+            } else {
+                (w.local_line_nr as f64).log10() as usize + 1
+            }
+        })
+        .max()
+        .unwrap_or(0);
+
+    let filename_len = words
+        .iter()
+        .filter(|w| w.filename != "-")
+        .map(|w| w.filename.maybe_quote().to_string().len())
+        .max()
+        .unwrap_or(0);
+
+    // +1 for the colon
+    line_num + filename_len + 1
+}
+
 mod options {
+    pub mod format {
+        pub static ROFF: &str = "roff";
+        pub static TEX: &str = "tex";
+    }
+
     pub static FILE: &str = "file";
     pub static AUTO_REFERENCE: &str = "auto-reference";
     pub static TRADITIONAL: &str = "traditional";
     pub static FLAG_TRUNCATION: &str = "flag-truncation";
     pub static MACRO_NAME: &str = "macro-name";
-    pub static FORMAT_ROFF: &str = "format=roff";
+    pub static FORMAT: &str = "format";
     pub static RIGHT_SIDE_REFS: &str = "right-side-refs";
     pub static SENTENCE_REGEXP: &str = "sentence-regexp";
-    pub static FORMAT_TEX: &str = "format=tex";
     pub static WORD_REGEXP: &str = "word-regexp";
     pub static BREAK_FILE: &str = "break-file";
     pub static IGNORE_CASE: &str = "ignore-case";
@@ -714,143 +901,174 @@ mod options {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().try_get_matches_from(args)?;
+    let mut matches = uucore::clap_localization::handle_clap_result(uu_app(), args)?;
+    let mut config = get_config(&mut matches)?;
 
-    let mut input_files: Vec<String> = match &matches.get_many::<String>(options::FILE) {
-        Some(v) => v.clone().cloned().collect(),
-        None => vec!["-".to_string()],
-    };
+    let input_files;
+    let output_file: OsString;
+
+    let mut files = matches
+        .get_many::<OsString>(options::FILE)
+        .into_iter()
+        .flatten()
+        .cloned();
+
+    if config.gnu_ext {
+        input_files = {
+            let mut files = files.collect::<Vec<_>>();
+            if files.is_empty() {
+                files.push(OsString::from("-"));
+            }
+            files
+        };
+        output_file = OsString::from("-");
+    } else {
+        input_files = vec![files.next().unwrap_or(OsString::from("-"))];
+        output_file = files.next().unwrap_or(OsString::from("-"));
+        if let Some(file) = files.next() {
+            return Err(UUsageError::new(
+                1,
+                translate!("ptx-error-extra-operand", "operand" => file.quote()),
+            ));
+        }
+    }
 
-    let config = get_config(&matches)?;
     let word_filter = WordFilter::new(&matches, &config)?;
     let file_map = read_input(&input_files, &config).map_err_context(String::new)?;
     let word_set = create_word_set(&config, &word_filter, &file_map);
-    let output_file = if !config.gnu_ext && input_files.len() == 2 {
-        input_files.pop().unwrap()
-    } else {
-        "-".to_string()
-    };
-    write_traditional_output(&config, &file_map, &word_set, &output_file)
+    write_traditional_output(&mut config, &file_map, &word_set, &output_file)
 }
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .about(ABOUT)
-        .version(crate_version!())
-        .override_usage(format_usage(USAGE))
+        .about(translate!("ptx-about"))
+        .version(uucore::crate_version!())
+        .help_template(uucore::localized_help_template(uucore::util_name()))
+        .override_usage(format_usage(&translate!("ptx-usage")))
         .infer_long_args(true)
         .arg(
             Arg::new(options::FILE)
                 .hide(true)
                 .action(ArgAction::Append)
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::AUTO_REFERENCE)
                 .short('A')
                 .long(options::AUTO_REFERENCE)
-                .help("output automatically generated references")
+                .help(translate!("ptx-help-auto-reference"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::TRADITIONAL)
                 .short('G')
                 .long(options::TRADITIONAL)
-                .help("behave more like System V 'ptx'")
+                .help(translate!("ptx-help-traditional"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FLAG_TRUNCATION)
                 .short('F')
                 .long(options::FLAG_TRUNCATION)
-                .help("use STRING for flagging line truncations")
+                .help(translate!("ptx-help-flag-truncation"))
                 .value_name("STRING"),
         )
         .arg(
             Arg::new(options::MACRO_NAME)
                 .short('M')
                 .long(options::MACRO_NAME)
-                .help("macro name to use instead of 'xx'")
+                .help(translate!("ptx-help-macro-name"))
                 .value_name("STRING"),
         )
         .arg(
-            Arg::new(options::FORMAT_ROFF)
+            Arg::new(options::FORMAT)
+                .long(options::FORMAT)
+                .hide(true)
+                .value_parser(["roff", "tex"])
+                .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX]),
+        )
+        .arg(
+            Arg::new(options::format::ROFF)
                 .short('O')
-                .long(options::FORMAT_ROFF)
-                .help("generate output as roff directives")
+                .help(translate!("ptx-help-roff"))
+                .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX])
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::format::TEX)
+                .short('T')
+                .help(translate!("ptx-help-tex"))
+                .overrides_with_all([options::FORMAT, options::format::ROFF, options::format::TEX])
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::RIGHT_SIDE_REFS)
                 .short('R')
                 .long(options::RIGHT_SIDE_REFS)
-                .help("put references at right, not counted in -w")
+                .help(translate!("ptx-help-right-side-refs"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SENTENCE_REGEXP)
                 .short('S')
                 .long(options::SENTENCE_REGEXP)
-                .help("for end of lines or end of sentences")
+                .help(translate!("ptx-help-sentence-regexp"))
                 .value_name("REGEXP"),
         )
-        .arg(
-            Arg::new(options::FORMAT_TEX)
-                .short('T')
-                .long(options::FORMAT_TEX)
-                .help("generate output as TeX directives")
-                .action(ArgAction::SetTrue),
-        )
         .arg(
             Arg::new(options::WORD_REGEXP)
                 .short('W')
                 .long(options::WORD_REGEXP)
-                .help("use REGEXP to match each keyword")
+                .help(translate!("ptx-help-word-regexp"))
                 .value_name("REGEXP"),
         )
         .arg(
             Arg::new(options::BREAK_FILE)
                 .short('b')
                 .long(options::BREAK_FILE)
-                .help("word break characters in this FILE")
+                .help(translate!("ptx-help-break-file"))
                 .value_name("FILE")
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::IGNORE_CASE)
                 .short('f')
                 .long(options::IGNORE_CASE)
-                .help("fold lower case to upper case for sorting")
+                .help(translate!("ptx-help-ignore-case"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::GAP_SIZE)
                 .short('g')
                 .long(options::GAP_SIZE)
-                .help("gap size in columns between output fields")
+                .help(translate!("ptx-help-gap-size"))
                 .value_name("NUMBER"),
         )
         .arg(
             Arg::new(options::IGNORE_FILE)
                 .short('i')
                 .long(options::IGNORE_FILE)
-                .help("read ignore word list from FILE")
+                .help(translate!("ptx-help-ignore-file"))
                 .value_name("FILE")
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::ONLY_FILE)
                 .short('o')
                 .long(options::ONLY_FILE)
-                .help("read only word list from this FILE")
+                .help(translate!("ptx-help-only-file"))
                 .value_name("FILE")
-                .value_hint(clap::ValueHint::FilePath),
+                .value_hint(clap::ValueHint::FilePath)
+                .value_parser(clap::value_parser!(OsString)),
         )
         .arg(
             Arg::new(options::REFERENCES)
                 .short('r')
                 .long(options::REFERENCES)
-                .help("first field of each line is a reference")
+                .help(translate!("ptx-help-references"))
                 .value_name("FILE")
                 .action(ArgAction::SetTrue),
         )
@@ -858,7 +1076,7 @@ pub fn uu_app() -> Command {
             Arg::new(options::WIDTH)
                 .short('w')
                 .long(options::WIDTH)
-                .help("output width in columns, reference excluded")
+                .help(translate!("ptx-help-width"))
                 .value_name("NUMBER"),
         )
 }
