@@ -3,24 +3,22 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-// spell-checker:ignore (ToDO) kqueue Signum fundu
+// spell-checker:ignore (ToDO) kqueue Signum
 
 use crate::paths::Input;
-use crate::{parse, platform, Quotable};
-use clap::{crate_version, value_parser};
-use clap::{Arg, ArgAction, ArgMatches, Command};
-use fundu::{DurationParser, SaturatingInto};
+use crate::{Quotable, parse, platform};
+use clap::{Arg, ArgAction, ArgMatches, Command, value_parser};
 use same_file::Handle;
 use std::ffi::OsString;
 use std::io::IsTerminal;
 use std::time::Duration;
 use uucore::error::{UResult, USimpleError, UUsageError};
-use uucore::parse_size::{parse_size_u64, ParseSizeError};
-use uucore::shortcut_value_parser::ShortcutValueParser;
-use uucore::{format_usage, help_about, help_usage, show_warning};
-
-const ABOUT: &str = help_about!("tail.md");
-const USAGE: &str = help_usage!("tail.md");
+use uucore::parser::parse_signed_num::{SignPrefix, parse_signed_num_max};
+use uucore::parser::parse_size::ParseSizeError;
+use uucore::parser::parse_time;
+use uucore::parser::shortcut_value_parser::ShortcutValueParser;
+use uucore::translate;
+use uucore::{format_usage, show_warning};
 
 pub mod options {
     pub mod verbosity {
@@ -40,6 +38,7 @@ pub mod options {
     pub const MAX_UNCHANGED_STATS: &str = "max-unchanged-stats";
     pub const ARG_FILES: &str = "files";
     pub const PRESUME_INPUT_PIPE: &str = "-presume-input-pipe"; // NOTE: three hyphens is correct
+    pub const DEBUG: &str = "debug";
 }
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
@@ -80,8 +79,8 @@ impl FilterMode {
                 Err(e) => {
                     return Err(USimpleError::new(
                         1,
-                        format!("invalid number of bytes: '{e}'"),
-                    ))
+                        translate!("tail-error-invalid-number-of-bytes", "arg" => e.to_string()),
+                    ));
                 }
             }
         } else if let Some(arg) = matches.get_one::<String>(options::LINES) {
@@ -90,11 +89,11 @@ impl FilterMode {
                     let delimiter = if zero_term { 0 } else { b'\n' };
                     Self::Lines(signum, delimiter)
                 }
-                Err(e) => {
+                Err(_) => {
                     return Err(USimpleError::new(
                         1,
-                        format!("invalid number of lines: {e}"),
-                    ))
+                        translate!("tail-error-invalid-number-of-lines", "arg" => arg.quote()),
+                    ));
                 }
             }
         } else if zero_term {
@@ -141,6 +140,7 @@ pub struct Settings {
     pub use_polling: bool,
     pub verbose: bool,
     pub presume_input_pipe: bool,
+    pub debug: bool,
     /// `FILE(s)` positional arguments
     pub inputs: Vec<Input>,
 }
@@ -157,6 +157,7 @@ impl Default for Settings {
             use_polling: Default::default(),
             verbose: Default::default(),
             presume_input_pipe: Default::default(),
+            debug: Default::default(),
             inputs: Vec::default(),
         }
     }
@@ -182,7 +183,7 @@ impl Settings {
         settings
     }
 
-    pub fn from(matches: &clap::ArgMatches) -> UResult<Self> {
+    pub fn from(matches: &ArgMatches) -> UResult<Self> {
         // We're parsing --follow, -F and --retry under the following conditions:
         // * -F sets --retry and --follow=name
         // * plain --follow or short -f is the same like specifying --follow=descriptor
@@ -225,26 +226,17 @@ impl Settings {
             mode: FilterMode::from(matches)?,
             verbose: matches.get_flag(options::verbosity::VERBOSE),
             presume_input_pipe: matches.get_flag(options::PRESUME_INPUT_PIPE),
+            debug: matches.get_flag(options::DEBUG),
             ..Default::default()
         };
 
         if let Some(source) = matches.get_one::<String>(options::SLEEP_INT) {
-            // Advantage of `fundu` over `Duration::(try_)from_secs_f64(source.parse().unwrap())`:
-            // * doesn't panic on errors like `Duration::from_secs_f64` would.
-            // * no precision loss, rounding errors or other floating point problems.
-            // * evaluates to `Duration::MAX` if the parsed number would have exceeded
-            //   `DURATION::MAX` or `infinity` was given
-            // * not applied here but it supports customizable time units and provides better error
-            //   messages
-            settings.sleep_sec = match DurationParser::without_time_units().parse(source) {
-                Ok(duration) => SaturatingInto::<std::time::Duration>::saturating_into(duration),
-                Err(_) => {
-                    return Err(UUsageError::new(
-                        1,
-                        format!("invalid number of seconds: '{source}'"),
-                    ))
-                }
-            }
+            settings.sleep_sec = parse_time::from_str(source, false).map_err(|_| {
+                UUsageError::new(
+                    1,
+                    translate!("tail-error-invalid-number-of-seconds", "source" => source.clone()),
+                )
+            })?;
         }
 
         if let Some(s) = matches.get_one::<String>(options::MAX_UNCHANGED_STATS) {
@@ -253,10 +245,7 @@ impl Settings {
                 Err(_) => {
                     return Err(UUsageError::new(
                         1,
-                        format!(
-                            "invalid maximum number of unchanged stats between opens: {}",
-                            s.quote()
-                        ),
+                        translate!("tail-error-invalid-max-unchanged-stats", "value" => s.quote()),
                     ));
                 }
             }
@@ -271,7 +260,7 @@ impl Settings {
                         // NOTE: tail only accepts an unsigned pid
                         return Err(USimpleError::new(
                             1,
-                            format!("invalid PID: {}", pid_str.quote()),
+                            translate!("tail-error-invalid-pid", "pid" => pid_str.quote()),
                         ));
                     }
 
@@ -280,7 +269,7 @@ impl Settings {
                 Err(e) => {
                     return Err(USimpleError::new(
                         1,
-                        format!("invalid PID: {}: {}", pid_str.quote(), e),
+                        translate!("tail-error-invalid-pid-with-error", "pid" => pid_str.quote(), "error" => e),
                     ));
                 }
             }
@@ -288,11 +277,11 @@ impl Settings {
 
         settings.inputs = matches
             .get_many::<OsString>(options::ARG_FILES)
-            .map(|v| v.map(Input::from).collect())
-            .unwrap_or_else(|| vec![Input::default()]);
+            .map_or_else(|| vec![Input::default()], |v| v.map(Input::from).collect());
 
-        settings.verbose =
-            settings.inputs.len() > 1 && !matches.get_flag(options::verbosity::QUIET);
+        settings.verbose = (matches.get_flag(options::verbosity::VERBOSE)
+            || settings.inputs.len() > 1)
+            && !matches.get_flag(options::verbosity::QUIET);
 
         Ok(settings)
     }
@@ -314,17 +303,17 @@ impl Settings {
     pub fn check_warnings(&self) {
         if self.retry {
             if self.follow.is_none() {
-                show_warning!("--retry ignored; --retry is useful only when following");
+                show_warning!("{}", translate!("tail-warning-retry-ignored"));
             } else if self.follow == Some(FollowMode::Descriptor) {
-                show_warning!("--retry only effective for the initial open");
+                show_warning!("{}", translate!("tail-warning-retry-only-effective"));
             }
         }
 
         if self.pid != 0 {
             if self.follow.is_none() {
-                show_warning!("PID ignored; --pid=PID is useful only when following");
+                show_warning!("{}", translate!("tail-warning-pid-ignored"));
             } else if !platform::supports_pid_checks(self.pid) {
-                show_warning!("--pid=PID is not supported on this system");
+                show_warning!("{}", translate!("tail-warning-pid-not-supported"));
             }
         }
 
@@ -344,7 +333,7 @@ impl Settings {
                 });
 
             if !blocking_stdin && std::io::stdin().is_terminal() {
-                show_warning!("following standard input indefinitely is ineffective");
+                show_warning!("{}", translate!("tail-warning-following-stdin-ineffective"));
             }
         }
     }
@@ -378,22 +367,18 @@ pub fn parse_obsolete(arg: &OsString, input: Option<&OsString>) -> UResult<Optio
         Some(Ok(args)) => Ok(Some(Settings::from_obsolete_args(&args, input))),
         None => Ok(None),
         Some(Err(e)) => {
-            let arg_str = arg.to_string_lossy();
             Err(USimpleError::new(
                 1,
                 match e {
-                    parse::ParseError::OutOfRange => format!(
-                        "invalid number: {}: Numerical result out of range",
-                        arg_str.quote()
-                    ),
-                    parse::ParseError::Overflow => format!("invalid number: {}", arg_str.quote()),
                     // this ensures compatibility to GNU's error message (as tested in misc/tail)
-                    parse::ParseError::Context => format!(
-                        "option used in invalid context -- {}",
-                        arg_str.chars().nth(1).unwrap_or_default()
-                    ),
+                    parse::ParseError::Context => {
+                        translate!(
+                            "tail-error-option-used-in-invalid-context",
+                            "option" => arg.to_string_lossy().chars().nth(1).unwrap_or_default(),
+                        )
+                    }
                     parse::ParseError::InvalidEncoding => {
-                        format!("bad argument encoding: '{arg_str}'")
+                        translate!("tail-error-bad-argument-encoding", "arg" => arg.quote())
                     }
                 },
             ))
@@ -402,27 +387,15 @@ pub fn parse_obsolete(arg: &OsString, input: Option<&OsString>) -> UResult<Optio
 }
 
 fn parse_num(src: &str) -> Result<Signum, ParseSizeError> {
-    let mut size_string = src.trim();
-    let mut starting_with = false;
+    let result = parse_signed_num_max(src)?;
+    // tail: '+' means "starting from line/byte N", default/'-' means "last N"
+    let is_plus = result.sign == Some(SignPrefix::Plus);
 
-    if let Some(c) = size_string.chars().next() {
-        if c == '+' || c == '-' {
-            // tail: '-' is not documented (8.32 man pages)
-            size_string = &size_string[1..];
-            if c == '+' {
-                starting_with = true;
-            }
-        }
-    }
-
-    match parse_size_u64(size_string) {
-        Ok(n) => match (n, starting_with) {
-            (0, true) => Ok(Signum::PlusZero),
-            (0, false) => Ok(Signum::MinusZero),
-            (n, true) => Ok(Signum::Positive(n)),
-            (n, false) => Ok(Signum::Negative(n)),
-        },
-        Err(_) => Err(ParseSizeError::ParseFailure(size_string.to_string())),
+    match (result.value, is_plus) {
+        (0, true) => Ok(Signum::PlusZero),
+        (0, false) => Ok(Signum::MinusZero),
+        (n, true) => Ok(Signum::Positive(n)),
+        (n, false) => Ok(Signum::Negative(n)),
     }
 }
 
@@ -469,16 +442,17 @@ pub fn parse_args(args: impl uucore::Args) -> UResult<Settings> {
 
 pub fn uu_app() -> Command {
     #[cfg(target_os = "linux")]
-    const POLLING_HELP: &str = "Disable 'inotify' support and use polling instead";
+    let polling_help = translate!("tail-help-polling-linux");
     #[cfg(all(unix, not(target_os = "linux")))]
-    const POLLING_HELP: &str = "Disable 'kqueue' support and use polling instead";
+    let polling_help = translate!("tail-help-polling-unix");
     #[cfg(target_os = "windows")]
-    const POLLING_HELP: &str = "Disable 'ReadDirectoryChanges' support and use polling instead";
+    let polling_help = translate!("tail-help-polling-windows");
 
     Command::new(uucore::util_name())
-        .version(crate_version!())
-        .about(ABOUT)
-        .override_usage(format_usage(USAGE))
+        .version(uucore::crate_version!())
+        .help_template(uucore::localized_help_template(uucore::util_name()))
+        .about(translate!("tail-about"))
+        .override_usage(format_usage(&translate!("tail-usage")))
         .infer_long_args(true)
         .arg(
             Arg::new(options::BYTES)
@@ -486,7 +460,7 @@ pub fn uu_app() -> Command {
                 .long(options::BYTES)
                 .allow_hyphen_values(true)
                 .overrides_with_all([options::BYTES, options::LINES])
-                .help("Number of bytes to print"),
+                .help(translate!("tail-help-bytes")),
         )
         .arg(
             Arg::new(options::FOLLOW)
@@ -497,7 +471,7 @@ pub fn uu_app() -> Command {
                 .require_equals(true)
                 .value_parser(ShortcutValueParser::new(["descriptor", "name"]))
                 .overrides_with(options::FOLLOW)
-                .help("Print the file as it grows"),
+                .help(translate!("tail-help-follow")),
         )
         .arg(
             Arg::new(options::LINES)
@@ -505,13 +479,13 @@ pub fn uu_app() -> Command {
                 .long(options::LINES)
                 .allow_hyphen_values(true)
                 .overrides_with_all([options::BYTES, options::LINES])
-                .help("Number of lines to print"),
+                .help(translate!("tail-help-lines")),
         )
         .arg(
             Arg::new(options::PID)
                 .long(options::PID)
                 .value_name("PID")
-                .help("With -f, terminate after process ID, PID dies")
+                .help(translate!("tail-help-pid"))
                 .overrides_with(options::PID),
         )
         .arg(
@@ -520,7 +494,7 @@ pub fn uu_app() -> Command {
                 .long(options::verbosity::QUIET)
                 .visible_alias("silent")
                 .overrides_with_all([options::verbosity::QUIET, options::verbosity::VERBOSE])
-                .help("Never output headers giving file names")
+                .help(translate!("tail-help-quiet"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -528,32 +502,27 @@ pub fn uu_app() -> Command {
                 .short('s')
                 .value_name("N")
                 .long(options::SLEEP_INT)
-                .help("Number of seconds to sleep between polling the file when running with -f"),
+                .help(translate!("tail-help-sleep-interval")),
         )
         .arg(
             Arg::new(options::MAX_UNCHANGED_STATS)
                 .value_name("N")
                 .long(options::MAX_UNCHANGED_STATS)
-                .help(
-                    "Reopen a FILE which has not changed size after N (default 5) iterations \
-                        to see if it has been unlinked or renamed (this is the usual case of rotated \
-                        log files); This option is meaningful only when polling \
-                        (i.e., with --use-polling) and when --follow=name",
-                ),
+                .help(translate!("tail-help-max-unchanged-stats")),
         )
         .arg(
             Arg::new(options::verbosity::VERBOSE)
                 .short('v')
                 .long(options::verbosity::VERBOSE)
                 .overrides_with_all([options::verbosity::QUIET, options::verbosity::VERBOSE])
-                .help("Always output headers giving file names")
+                .help(translate!("tail-help-verbose"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::ZERO_TERM)
                 .short('z')
                 .long(options::ZERO_TERM)
-                .help("Line delimiter is NUL, not newline")
+                .help(translate!("tail-help-zero-terminated"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -561,23 +530,29 @@ pub fn uu_app() -> Command {
                 .alias(options::DISABLE_INOTIFY_TERM) // NOTE: Used by GNU's test suite
                 .alias("dis") // NOTE: Used by GNU's test suite
                 .long(options::USE_POLLING)
-                .help(POLLING_HELP)
+                .help(polling_help)
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::RETRY)
                 .long(options::RETRY)
-                .help("Keep trying to open a file if it is inaccessible")
+                .help(translate!("tail-help-retry"))
                 .overrides_with(options::RETRY)
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FOLLOW_RETRY)
                 .short('F')
-                .help("Same as --follow=name --retry")
+                .help(translate!("tail-help-follow-retry"))
                 .overrides_with(options::FOLLOW_RETRY)
                 .action(ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new(options::DEBUG)
+                .long(options::DEBUG)
+                .help(translate!("tail-help-debug"))
+                .action(ArgAction::SetTrue),
+        )
         .arg(
             Arg::new(options::PRESUME_INPUT_PIPE)
                 .long("presume-input-pipe")
