@@ -40,9 +40,9 @@ pub enum Symbol {
 }
 
 impl Symbol {
-    /// Create a new Symbol from an OsString.
+    /// Create a new Symbol from an [`OsString`].
     ///
-    /// Returns Symbol::None in place of None
+    /// Returns `Symbol::None` in place of None
     fn new(token: Option<OsString>) -> Self {
         match token {
             Some(s) => match s.to_str() {
@@ -66,24 +66,21 @@ impl Symbol {
         }
     }
 
-    /// Convert this Symbol into a Symbol::Literal, useful for cases where
+    /// Convert this Symbol into a [`Symbol::Literal`], useful for cases where
     /// test treats an operator as a string operand (test has no reserved
     /// words).
     ///
     /// # Panics
     ///
-    /// Panics if `self` is Symbol::None
+    /// Panics if `self` is [`Symbol::None`]
     fn into_literal(self) -> Self {
         Self::Literal(match self {
             Self::LParen => OsString::from("("),
             Self::Bang => OsString::from("!"),
             Self::BoolOp(s)
             | Self::Literal(s)
-            | Self::Op(Operator::String(s))
-            | Self::Op(Operator::Int(s))
-            | Self::Op(Operator::File(s))
-            | Self::UnaryOp(UnaryOperator::StrlenOp(s))
-            | Self::UnaryOp(UnaryOperator::FiletestOp(s)) => s,
+            | Self::Op(Operator::String(s) | Operator::Int(s) | Operator::File(s))
+            | Self::UnaryOp(UnaryOperator::StrlenOp(s) | UnaryOperator::FiletestOp(s)) => s,
             Self::None => panic!(),
         })
     }
@@ -99,18 +96,17 @@ impl std::fmt::Display for Symbol {
             Self::Bang => OsStr::new("!"),
             Self::BoolOp(s)
             | Self::Literal(s)
-            | Self::Op(Operator::String(s))
-            | Self::Op(Operator::Int(s))
-            | Self::Op(Operator::File(s))
-            | Self::UnaryOp(UnaryOperator::StrlenOp(s))
-            | Self::UnaryOp(UnaryOperator::FiletestOp(s)) => OsStr::new(s),
+            | Self::Op(Operator::String(s) | Operator::Int(s) | Operator::File(s))
+            | Self::UnaryOp(UnaryOperator::StrlenOp(s) | UnaryOperator::FiletestOp(s)) => {
+                OsStr::new(s)
+            }
             Self::None => OsStr::new("None"),
         };
         write!(f, "{}", s.quote())
     }
 }
 
-/// Recursive descent parser for test, which converts a list of OsStrings
+/// Recursive descent parser for test, which converts a list of [`OsString`]s
 /// (typically command line arguments) into a stack of Symbols in postfix
 /// order.
 ///
@@ -192,7 +188,16 @@ impl Parser {
         match symbol {
             Symbol::LParen => self.lparen()?,
             Symbol::Bang => self.bang()?,
-            Symbol::UnaryOp(_) => self.uop(symbol),
+            Symbol::UnaryOp(_) => {
+                // Three-argument string comparison: `-f = a` means "-f" = "a", not file test
+                let is_string_cmp = matches!(self.peek(), Symbol::Op(Operator::String(_)))
+                    && !matches!(Symbol::new(self.tokens.clone().nth(1)), Symbol::None);
+                if is_string_cmp {
+                    self.literal(symbol.into_literal())?;
+                } else {
+                    self.uop(symbol);
+                }
+            }
             Symbol::None => self.stack.push(symbol),
             literal => self.literal(literal)?,
         }
