@@ -9,22 +9,23 @@ use std::path::PathBuf;
 use thiserror::Error;
 use uucore::display::Quotable;
 use uucore::error::{UError, UIoError};
+use uucore::translate;
 
 #[derive(Debug, Error)]
 pub enum TouchError {
-    #[error("Unable to parse date: {0}")]
+    #[error("{}", translate!("touch-error-unable-to-parse-date", "date" => .0.clone()))]
     InvalidDateFormat(String),
 
-    /// The source time couldn't be converted to a [chrono::DateTime]
-    #[error("Source has invalid access or modification time: {0}")]
+    /// The source time couldn't be converted to a [`jiff::Zoned`]
+    #[error("{}", translate!("touch-error-invalid-filetime", "time" => .0))]
     InvalidFiletime(FileTime),
 
     /// The reference file's attributes could not be found or read
-    #[error("failed to get attributes of {}: {}", .0.quote(), to_uioerror(.1))]
+    #[error("{}", translate!("touch-error-reference-file-inaccessible", "path" => .0.quote(), "error" => to_uioerror(.1)))]
     ReferenceFileInaccessible(PathBuf, std::io::Error),
 
     /// An error getting a path to stdout on Windows
-    #[error("GetFinalPathNameByHandleW failed with code {0}")]
+    #[error("{}", translate!("touch-error-windows-stdout-path-failed", "code" => .0.clone()))]
     WindowsStdoutPathError(String),
 
     /// An error encountered on a specific file
