@@ -13,23 +13,26 @@ mod word_count;
 use std::{
     borrow::{Borrow, Cow},
     cmp::max,
+    env,
     ffi::{OsStr, OsString},
     fs::{self, File},
-    io::{self, Write},
+    io::{self, Write, stderr},
     iter,
     path::{Path, PathBuf},
 };
 
-use clap::{builder::ValueParser, crate_version, Arg, ArgAction, ArgMatches, Command};
+use clap::{Arg, ArgAction, ArgMatches, Command, builder::ValueParser};
 use thiserror::Error;
 use unicode_width::UnicodeWidthChar;
 use utf8::{BufReadDecoder, BufReadDecoderError};
+use uucore::{display::Quotable, translate};
 
 use uucore::{
     error::{FromIo, UError, UResult},
-    format_usage, help_about, help_usage,
+    format_usage,
+    hardware::{HardwareFeature, HasHardwareFeatures as _, SimdPolicy},
+    parser::shortcut_value_parser::ShortcutValueParser,
     quoting_style::{self, QuotingStyle},
-    shortcut_value_parser::ShortcutValueParser,
     show,
 };
 
@@ -48,6 +51,7 @@ struct Settings<'a> {
     show_lines: bool,
     show_words: bool,
     show_max_line_length: bool,
+    debug: bool,
     files0_from: Option<Input<'a>>,
     total_when: TotalWhen,
 }
@@ -61,6 +65,7 @@ impl Default for Settings<'_> {
             show_lines: true,
             show_words: true,
             show_max_line_length: false,
+            debug: false,
             files0_from: None,
             total_when: TotalWhen::default(),
         }
@@ -84,6 +89,7 @@ impl<'a> Settings<'a> {
             show_lines: matches.get_flag(options::LINES),
             show_words: matches.get_flag(options::WORDS),
             show_max_line_length: matches.get_flag(options::MAX_LINE_LENGTH),
+            debug: matches.get_flag(options::DEBUG),
             files0_from,
             total_when,
         };
@@ -94,6 +100,7 @@ impl<'a> Settings<'a> {
             Self {
                 files0_from: settings.files0_from,
                 total_when,
+                debug: settings.debug,
                 ..Default::default()
             }
         }
@@ -113,9 +120,6 @@ impl<'a> Settings<'a> {
     }
 }
 
-const ABOUT: &str = help_about!("wc.md");
-const USAGE: &str = help_usage!("wc.md");
-
 mod options {
     pub static BYTES: &str = "bytes";
     pub static CHAR: &str = "chars";
@@ -124,21 +128,11 @@ mod options {
     pub static MAX_LINE_LENGTH: &str = "max-line-length";
     pub static TOTAL: &str = "total";
     pub static WORDS: &str = "words";
+    pub static DEBUG: &str = "debug";
 }
 static ARG_FILES: &str = "files";
 static STDIN_REPR: &str = "-";
 
-static QS_ESCAPE: &QuotingStyle = &QuotingStyle::Shell {
-    escape: true,
-    always_quote: false,
-    show_control: false,
-};
-static QS_QUOTE_ESCAPE: &QuotingStyle = &QuotingStyle::Shell {
-    escape: true,
-    always_quote: true,
-    show_control: false,
-};
-
 /// Supported inputs.
 #[derive(Debug)]
 enum Inputs<'a> {
@@ -174,8 +168,8 @@ impl<'a> Inputs<'a> {
         }
     }
 
-    // Creates an iterator which yields values borrowed from the command line arguments.
-    // Returns an error if the file specified in --files0-from cannot be opened.
+    /// Creates an iterator which yields values borrowed from the command line arguments.
+    /// Returns an error if the file specified in --files0-from cannot be opened.
     fn try_iter(
         &'a self,
         settings: &'a Settings<'a>,
@@ -210,7 +204,7 @@ impl<'a> Inputs<'a> {
 
 #[derive(Clone, Copy, Debug)]
 enum StdinKind {
-    /// Specified on command-line with "-" (STDIN_REPR)
+    /// Specified on command-line with "-" ([`STDIN_REPR`])
     Explicit,
     /// Implied by the lack of any arguments
     Implicit,
@@ -246,7 +240,7 @@ impl<'a, T: AsRef<Path> + ?Sized> From<&'a T> for Input<'a> {
 }
 
 impl<'a> Input<'a> {
-    /// Translates Path(Cow::Owned(_)) to Path(Cow::Borrowed(_)).
+    /// Translates `Path(Cow::Owned(_))` to `Path(Cow::Borrowed(_))`.
     fn as_borrowed(&'a self) -> Self {
         match self {
             Self::Path(p) => Self::Path(Cow::Borrowed(p.borrow())),
@@ -255,12 +249,15 @@ impl<'a> Input<'a> {
     }
 
     /// Converts input to title that appears in stats.
-    fn to_title(&self) -> Option<Cow<OsStr>> {
+    fn to_title(&self) -> Option<Cow<'_, OsStr>> {
         match self {
             Self::Path(path) => {
                 let path = path.as_os_str();
                 if path.to_string_lossy().contains('\n') {
-                    Some(Cow::Owned(quoting_style::escape_name(path, QS_ESCAPE)))
+                    Some(Cow::Owned(quoting_style::locale_aware_escape_name(
+                        path,
+                        QuotingStyle::SHELL_ESCAPE,
+                    )))
                 } else {
                     Some(Cow::Borrowed(path))
                 }
@@ -274,13 +271,13 @@ impl<'a> Input<'a> {
     fn path_display(&self) -> String {
         match self {
             Self::Path(path) => escape_name_wrapper(path.as_os_str()),
-            Self::Stdin(_) => String::from("standard input"),
+            Self::Stdin(_) => translate!("wc-standard-input"),
         }
     }
 
     /// When given --files0-from, we may be given a path or stdin. Either may be a stream or
     /// a regular file. If given a file less than 10 MiB, it will be consumed and turned into
-    /// a Vec of Input::Paths which can be scanned to determine the widths of the columns that
+    /// a Vec of [`Input::Path`] which can be scanned to determine the widths of the columns that
     /// will ultimately be printed.
     fn try_as_files0(&self) -> UResult<Option<Vec<Input<'static>>>> {
         match self {
@@ -309,8 +306,8 @@ fn is_stdin_small_file() -> bool {
 }
 
 #[cfg(not(unix))]
-// Windows presents a piped stdin as a "normal file" with a length equal to however many bytes
-// have been buffered at the time it's checked. To be safe, we must never assume it's a file.
+/// Windows presents a piped stdin as a "normal file" with a length equal to however many bytes
+/// have been buffered at the time it's checked. To be safe, we must never assume it's a file.
 fn is_stdin_small_file() -> bool {
     false
 }
@@ -349,13 +346,13 @@ impl TotalWhen {
 
 #[derive(Debug, Error)]
 enum WcError {
-    #[error("extra operand '{extra}'\nfile operands cannot be combined with --files0-from")]
-    FilesDisabled { extra: Cow<'static, str> },
-    #[error("when reading file names from stdin, no file name of '-' allowed")]
+    #[error("{}", translate!("wc-error-files-disabled", "extra" => extra.quote()))]
+    FilesDisabled { extra: Cow<'static, OsStr> },
+    #[error("{}", translate!("wc-error-stdin-repr-not-allowed"))]
     StdinReprNotAllowed,
-    #[error("invalid zero-length file name")]
+    #[error("{}", translate!("wc-error-zero-length-filename"))]
     ZeroLengthFileName,
-    #[error("{path}:{idx}: invalid zero-length file name")]
+    #[error("{}", translate!("wc-error-zero-length-filename-ctx", "path" => path, "idx" => idx))]
     ZeroLengthFileNameCtx { path: Cow<'static, str>, idx: usize },
 }
 
@@ -373,7 +370,7 @@ impl WcError {
         }
     }
     fn files_disabled(first_extra: &OsString) -> Self {
-        let extra = first_extra.to_string_lossy().into_owned().into();
+        let extra = first_extra.clone().into();
         Self::FilesDisabled { extra }
     }
 }
@@ -386,7 +383,7 @@ impl UError for WcError {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().try_get_matches_from(args)?;
+    let matches = uucore::clap_localization::handle_clap_result(uu_app(), args)?;
 
     let settings = Settings::new(&matches);
     let inputs = Inputs::new(&matches)?;
@@ -396,34 +393,31 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .version(crate_version!())
-        .about(ABOUT)
-        .override_usage(format_usage(USAGE))
+        .version(uucore::crate_version!())
+        .help_template(uucore::localized_help_template(uucore::util_name()))
+        .about(translate!("wc-about"))
+        .override_usage(format_usage(&translate!("wc-usage")))
         .infer_long_args(true)
         .args_override_self(true)
         .arg(
             Arg::new(options::BYTES)
                 .short('c')
                 .long(options::BYTES)
-                .help("print the byte counts")
+                .help(translate!("wc-help-bytes"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::CHAR)
                 .short('m')
                 .long(options::CHAR)
-                .help("print the character counts")
+                .help(translate!("wc-help-chars"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FILES0_FROM)
                 .long(options::FILES0_FROM)
                 .value_name("F")
-                .help(concat!(
-                    "read input from the files specified by\n",
-                    "  NUL-terminated names in file F;\n",
-                    "  If F is - then read names from standard input"
-                ))
+                .help(translate!("wc-help-files0-from"))
                 .value_parser(ValueParser::os_string())
                 .value_hint(clap::ValueHint::FilePath),
         )
@@ -431,14 +425,14 @@ pub fn uu_app() -> Command {
             Arg::new(options::LINES)
                 .short('l')
                 .long(options::LINES)
-                .help("print the newline counts")
+                .help(translate!("wc-help-lines"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::MAX_LINE_LENGTH)
                 .short('L')
                 .long(options::MAX_LINE_LENGTH)
-                .help("print the length of the longest line")
+                .help(translate!("wc-help-max-line-length"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -449,18 +443,21 @@ pub fn uu_app() -> Command {
                 ]))
                 .value_name("WHEN")
                 .hide_possible_values(true)
-                .help(concat!(
-                    "when to print a line with total counts;\n",
-                    "  WHEN can be: auto, always, only, never"
-                )),
+                .help(translate!("wc-help-total")),
         )
         .arg(
             Arg::new(options::WORDS)
                 .short('w')
                 .long(options::WORDS)
-                .help("print the word counts")
+                .help(translate!("wc-help-words"))
                 .action(ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new(options::DEBUG)
+                .long(options::DEBUG)
+                .action(ArgAction::SetTrue)
+                .hide(true),
+        )
         .arg(
             Arg::new(ARG_FILES)
                 .action(ArgAction::Append)
@@ -582,10 +579,17 @@ fn process_chunk<
     text: &str,
     current_len: &mut usize,
     in_word: &mut bool,
+    posixly_correct: bool,
 ) {
     for ch in text.chars() {
         if SHOW_WORDS {
-            if ch.is_whitespace() {
+            let is_space = if posixly_correct {
+                matches!(ch, '\t'..='\r' | ' ')
+            } else {
+                ch.is_whitespace()
+            };
+
+            if is_space {
                 *in_word = false;
             } else if !(*in_word) {
                 // This also counts control characters! (As of GNU coreutils 9.5)
@@ -620,10 +624,18 @@ fn process_chunk<
     total.max_line_length = max(*current_len, total.max_line_length);
 }
 
-fn handle_error(error: BufReadDecoderError<'_>, total: &mut WordCount) -> Option<io::Error> {
+fn handle_error(
+    error: BufReadDecoderError<'_>,
+    total: &mut WordCount,
+    in_word: &mut bool,
+) -> Option<io::Error> {
     match error {
         BufReadDecoderError::InvalidByteSequence(bytes) => {
             total.bytes += bytes.len();
+            if !(*in_word) {
+                *in_word = true;
+                total.words += 1;
+            }
         }
         BufReadDecoderError::Io(e) => return Some(e),
     }
@@ -643,6 +655,7 @@ fn word_count_from_reader_specialized<
     let mut reader = BufReadDecoder::new(reader.buffered());
     let mut in_word = false;
     let mut current_len = 0;
+    let posixly_correct = env::var_os("POSIXLY_CORRECT").is_some();
     while let Some(chunk) = reader.next_strict() {
         match chunk {
             Ok(text) => {
@@ -651,10 +664,11 @@ fn word_count_from_reader_specialized<
                     text,
                     &mut current_len,
                     &mut in_word,
+                    posixly_correct,
                 );
             }
             Err(e) => {
-                if let Some(e) = handle_error(e, &mut total) {
+                if let Some(e) = handle_error(e, &mut total, &mut in_word) {
                     return (total, Some(e));
                 }
             }
@@ -673,11 +687,11 @@ enum CountResult {
     Failure(io::Error),
 }
 
-/// If we fail opening a file, we only show the error. If we fail reading the
+/// If we fail to open a file, we only show the error. If we fail reading the
 /// file, we show a count for what we managed to read.
 ///
 /// Therefore, the reading implementations always return a total and sometimes
-/// return an error: (WordCount, Option<io::Error>).
+/// return an error: ([`WordCount`], `Option<io::Error>`).
 fn word_count_from_input(input: &Input<'_>, settings: &Settings) -> CountResult {
     let (total, maybe_err) = match input {
         Input::Stdin(_) => word_count_from_reader(io::stdin().lock(), settings),
@@ -750,7 +764,7 @@ fn compute_number_width(inputs: &Inputs, settings: &Settings) -> usize {
 
 type InputIterItem<'a> = Result<Input<'a>, Box<dyn UError>>;
 
-/// To be used with `--files0-from=-`, this applies a filter on the results of files0_iter to
+/// To be used with `--files0-from=-`, this applies a filter on the results of [`files0_iter`] to
 /// translate '-' into the appropriate error.
 fn files0_iter_stdin<'a>() -> impl Iterator<Item = InputIterItem<'a>> {
     files0_iter(io::stdin().lock(), STDIN_REPR.into()).map(|i| match i {
@@ -763,11 +777,13 @@ fn files0_iter_file<'a>(path: &Path) -> UResult<impl Iterator<Item = InputIterIt
     match File::open(path) {
         Ok(f) => Ok(files0_iter(f, path.into())),
         Err(e) => Err(e.map_err_context(|| {
-            format!(
-                "cannot open {} for reading",
-                quoting_style::escape_name(path.as_os_str(), QS_QUOTE_ESCAPE)
-                    .into_string()
-                    .expect("All escaped names with the escaping option return valid strings.")
+            translate!("wc-error-cannot-open-for-reading",
+                "path" => quoting_style::locale_aware_escape_name(
+                    path.as_os_str(),
+                    QuotingStyle::SHELL_ESCAPE_QUOTE,
+                )
+                .into_string()
+                .expect("All escaped names with the escaping option return valid strings.")
             )
         })),
     }
@@ -794,18 +810,17 @@ fn files0_iter<'a>(
                     // ...Windows does not, we must go through Strings.
                     #[cfg(not(unix))]
                     {
-                        let s = String::from_utf8(p)
-                            .map_err(|e| io::Error::new(io::ErrorKind::Other, e))?;
+                        let s = String::from_utf8(p).map_err(io::Error::other)?;
                         Ok(Input::Path(PathBuf::from(s).into()))
                     }
                 }
-                Err(e) => Err(e
-                    .map_err_context(|| format!("{}: read error", escape_name_wrapper(&err_path)))
-                    as Box<dyn UError>),
+                Err(e) => Err(e.map_err_context(
+                    || translate!("wc-error-read-error", "path" => escape_name_wrapper(&err_path)),
+                ) as Box<dyn UError>),
             }),
     );
     // Loop until there is an error; yield that error and then nothing else.
-    std::iter::from_fn(move || {
+    iter::from_fn(move || {
         let next = i.as_mut().and_then(Iterator::next);
         if matches!(next, Some(Err(_)) | None) {
             i = None;
@@ -815,11 +830,79 @@ fn files0_iter<'a>(
 }
 
 fn escape_name_wrapper(name: &OsStr) -> String {
-    quoting_style::escape_name(name, QS_ESCAPE)
+    quoting_style::locale_aware_escape_name(name, QuotingStyle::SHELL_ESCAPE)
         .into_string()
         .expect("All escaped names with the escaping option return valid strings.")
 }
 
+fn hardware_feature_label(feature: HardwareFeature) -> &'static str {
+    match feature {
+        HardwareFeature::Avx512 => "AVX512F",
+        HardwareFeature::Avx2 => "AVX2",
+        HardwareFeature::PclMul => "PCLMUL",
+        HardwareFeature::Vmull => "VMULL",
+        HardwareFeature::Sse2 => "SSE2",
+        HardwareFeature::Asimd => "ASIMD",
+    }
+}
+
+fn is_simd_runtime_feature(feature: &HardwareFeature) -> bool {
+    matches!(
+        feature,
+        HardwareFeature::Avx2 | HardwareFeature::Sse2 | HardwareFeature::Asimd
+    )
+}
+
+fn is_simd_debug_feature(feature: &HardwareFeature) -> bool {
+    matches!(
+        feature,
+        HardwareFeature::Avx512
+            | HardwareFeature::Avx2
+            | HardwareFeature::Sse2
+            | HardwareFeature::Asimd
+    )
+}
+
+struct WcSimdFeatures {
+    enabled: Vec<HardwareFeature>,
+    disabled: Vec<HardwareFeature>,
+    disabled_runtime: Vec<HardwareFeature>,
+}
+
+fn wc_simd_features(policy: &SimdPolicy) -> WcSimdFeatures {
+    let enabled = policy
+        .iter_features()
+        .filter(is_simd_runtime_feature)
+        .collect();
+
+    let mut disabled = Vec::new();
+    let mut disabled_runtime = Vec::new();
+    for feature in policy.disabled_features() {
+        if is_simd_debug_feature(&feature) {
+            disabled.push(feature);
+        }
+        if is_simd_runtime_feature(&feature) {
+            disabled_runtime.push(feature);
+        }
+    }
+
+    WcSimdFeatures {
+        enabled,
+        disabled,
+        disabled_runtime,
+    }
+}
+
+pub(crate) fn wc_simd_allowed(policy: &SimdPolicy) -> bool {
+    let disabled_features = policy.disabled_features();
+    if disabled_features.iter().any(is_simd_runtime_feature) {
+        return false;
+    }
+    policy
+        .iter_features()
+        .any(|feature| is_simd_runtime_feature(&feature))
+}
+
 fn wc(inputs: &Inputs, settings: &Settings) -> UResult<()> {
     let mut total_word_count = WordCount::default();
     let mut num_inputs: usize = 0;
@@ -829,6 +912,54 @@ fn wc(inputs: &Inputs, settings: &Settings) -> UResult<()> {
         _ => (compute_number_width(inputs, settings), true),
     };
 
+    if settings.debug {
+        let policy = SimdPolicy::detect();
+        let features = wc_simd_features(policy);
+
+        let enabled: Vec<&'static str> = features
+            .enabled
+            .iter()
+            .copied()
+            .map(hardware_feature_label)
+            .collect();
+        let disabled: Vec<&'static str> = features
+            .disabled
+            .iter()
+            .copied()
+            .map(hardware_feature_label)
+            .collect();
+
+        let enabled_empty = enabled.is_empty();
+        let disabled_empty = disabled.is_empty();
+        let runtime_disabled = !features.disabled_runtime.is_empty();
+
+        if enabled_empty && !runtime_disabled {
+            let _ = writeln!(stderr(), "{}", translate!("wc-debug-hw-unavailable"));
+        } else if runtime_disabled {
+            let _ = writeln!(
+                stderr(),
+                "{}",
+                translate!("wc-debug-hw-disabled-glibc", "features" => disabled.join(", "))
+            );
+        } else if !enabled_empty && disabled_empty {
+            let _ = writeln!(
+                stderr(),
+                "{}",
+                translate!("wc-debug-hw-using", "features" => enabled.join(", "))
+            );
+        } else {
+            let _ = writeln!(
+                stderr(),
+                "{}",
+                translate!(
+                    "wc-debug-hw-limited-glibc",
+                    "disabled" => disabled.join(", "),
+                    "enabled" => enabled.join(", ")
+                )
+            );
+        }
+    }
+
     for maybe_input in inputs.try_iter(settings)? {
         num_inputs += 1;
 
@@ -840,12 +971,13 @@ fn wc(inputs: &Inputs, settings: &Settings) -> UResult<()> {
             }
         };
 
-        let word_count = match word_count_from_input(&input, settings) {
-            CountResult::Success(word_count) => word_count,
-            CountResult::Interrupted(word_count, err) => {
-                show!(err.map_err_context(|| input.path_display()));
-                word_count
-            }
+        // Store any I/O error from reading to print AFTER stats (matches GNU wc behavior)
+        let (word_count, deferred_error) = match word_count_from_input(&input, settings) {
+            CountResult::Success(word_count) => (word_count, None),
+            CountResult::Interrupted(word_count, err) => (
+                word_count,
+                Some(err.map_err_context(|| input.path_display())),
+            ),
             CountResult::Failure(err) => {
                 show!(err.map_err_context(|| input.path_display()));
                 continue;
@@ -857,18 +989,21 @@ fn wc(inputs: &Inputs, settings: &Settings) -> UResult<()> {
             let maybe_title_str = maybe_title.as_deref();
             if let Err(err) = print_stats(settings, &word_count, maybe_title_str, number_width) {
                 let title = maybe_title_str.unwrap_or(OsStr::new("<stdin>"));
-                show!(err.map_err_context(|| format!(
-                    "failed to print result for {}",
-                    title.to_string_lossy()
-                )));
+                show!(err.map_err_context(|| translate!("wc-error-failed-to-print-result", "title" => title.to_string_lossy())));
             }
         }
+        // Print deferred error after stats to match GNU wc output order
+        if let Some(err) = deferred_error {
+            let _ = io::stdout().flush();
+            show!(err);
+        }
     }
 
     if settings.total_when.is_total_row_visible(num_inputs) {
-        let title = are_stats_visible.then_some(OsStr::new("total"));
+        let wc_total_msg = translate!("wc-total");
+        let title = are_stats_visible.then_some(OsStr::new(&wc_total_msg));
         if let Err(err) = print_stats(settings, &total_word_count, title, number_width) {
-            show!(err.map_err_context(|| "failed to print total".into()));
+            show!(err.map_err_context(|| translate!("wc-error-failed-to-print-total")));
         }
     }
 
