@@ -2,12 +2,11 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore (words) bamf chdir rlimit prlimit COMSPEC cout cerr FFFD
+// spell-checker:ignore (words) bamf chdir rlimit prlimit COMSPEC cout cerr FFFD winsize xpixel ypixel Secho sighandler
 #![allow(clippy::missing_errors_doc)]
 
-use crate::common::util::TestScenario;
 #[cfg(unix)]
-use crate::common::util::UChild;
+use nix::libc;
 #[cfg(unix)]
 use nix::sys::signal::Signal;
 #[cfg(feature = "echo")]
@@ -17,6 +16,15 @@ use std::path::Path;
 #[cfg(unix)]
 use std::process::Command;
 use tempfile::tempdir;
+use uutests::new_ucmd;
+#[cfg(unix)]
+use uutests::util::PATH;
+#[cfg(unix)]
+use uutests::util::TerminalSimulation;
+use uutests::util::TestScenario;
+#[cfg(unix)]
+use uutests::util::UChild;
+use uutests::util_name;
 
 #[cfg(unix)]
 struct Target {
@@ -25,13 +33,13 @@ struct Target {
 #[cfg(unix)]
 impl Target {
     fn new(signals: &[&str]) -> Self {
-        let mut child = new_ucmd!()
-            .args(&[
-                format!("--ignore-signal={}", signals.join(",")).as_str(),
-                "sleep",
-                "1000",
-            ])
-            .run_no_wait();
+        let mut cmd = new_ucmd!();
+        if signals.is_empty() {
+            cmd.arg("--ignore-signal");
+        } else {
+            cmd.arg(format!("--ignore-signal={}", signals.join(",")));
+        }
+        let mut child = cmd.args(&["sleep", "1000"]).run_no_wait();
         child.delay(500);
         Self { child }
     }
@@ -62,6 +70,61 @@ fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails_with_code(125);
 }
 
+#[test]
+#[cfg(not(target_os = "windows"))]
+fn test_flags_after_command() {
+    new_ucmd!()
+        // This would cause an error if -u=v were processed because it's malformed
+        .args(&["echo", "-u=v"])
+        .succeeds()
+        .no_stderr()
+        .stdout_is("-u=v\n");
+
+    new_ucmd!()
+        // Ensure the string isn't split
+        // cSpell:disable
+        .args(&["printf", "%s-%s", "-Sfoo bar"])
+        .succeeds()
+        .no_stderr()
+        .stdout_is("-Sfoo bar-");
+    // cSpell:enable
+
+    new_ucmd!()
+        // Ensure -- is recognized
+        .args(&["-i", "--", "-u=v"])
+        .succeeds()
+        .no_stderr()
+        .stdout_is("-u=v\n");
+
+    new_ucmd!()
+        // Recognize echo as the command after a flag that takes a value
+        .args(&["-C", "..", "echo", "-u=v"])
+        .succeeds()
+        .no_stderr()
+        .stdout_is("-u=v\n");
+
+    new_ucmd!()
+        // Recognize echo as the command after a flag that takes an inline value
+        .args(&["-C..", "echo", "-u=v"])
+        .succeeds()
+        .no_stderr()
+        .stdout_is("-u=v\n");
+
+    new_ucmd!()
+        // Recognize echo as the command after a flag that takes a value after another flag
+        .args(&["-iC", "..", "echo", "-u=v"])
+        .succeeds()
+        .no_stderr()
+        .stdout_is("-u=v\n");
+
+    new_ucmd!()
+        // Similar to the last two combined
+        .args(&["-iC..", "echo", "-u=v"])
+        .succeeds()
+        .no_stderr()
+        .stdout_is("-u=v\n");
+}
+
 #[test]
 fn test_env_help() {
     new_ucmd!()
@@ -81,11 +144,13 @@ fn test_env_version() {
 }
 
 #[test]
+#[cfg(unix)]
 fn test_env_permissions() {
+    // Try to execute `empty` in test fixture, that does not have exec permission.
     new_ucmd!()
-        .arg(".")
+        .arg("./empty")
         .fails_with_code(126)
-        .stderr_is("env: '.': Permission denied\n");
+        .stderr_is("env: './empty': Permission denied\n");
 }
 
 #[test]
@@ -210,7 +275,7 @@ fn test_file_option() {
     let out = new_ucmd!()
         .arg("-f")
         .arg("vars.conf.txt")
-        .run()
+        .succeeds()
         .stdout_move_str();
 
     assert_eq!(
@@ -227,7 +292,7 @@ fn test_combined_file_set() {
         .arg("-f")
         .arg("vars.conf.txt")
         .arg("FOO=bar.alt")
-        .run()
+        .succeeds()
         .stdout_move_str();
 
     assert_eq!(out.lines().filter(|&line| line == "FOO=bar.alt").count(), 1);
@@ -259,7 +324,7 @@ fn test_unset_invalid_variables() {
     // Cannot test input with \0 in it, since output will also contain \0. rlimit::prlimit fails
     // with this error: Error { kind: InvalidInput, message: "nul byte found in provided data" }
     for var in ["", "a=b"] {
-        new_ucmd!().arg("-u").arg(var).run().stderr_only(format!(
+        new_ucmd!().arg("-u").arg(var).fails().stderr_only(format!(
             "env: cannot unset {}: Invalid argument\n",
             var.quote()
         ));
@@ -268,14 +333,17 @@ fn test_unset_invalid_variables() {
 
 #[test]
 fn test_single_name_value_pair() {
-    let out = new_ucmd!().arg("FOO=bar").run();
-
-    assert!(out.stdout_str().lines().any(|line| line == "FOO=bar"));
+    new_ucmd!()
+        .arg("FOO=bar")
+        .succeeds()
+        .stdout_str()
+        .lines()
+        .any(|line| line == "FOO=bar");
 }
 
 #[test]
 fn test_multiple_name_value_pairs() {
-    let out = new_ucmd!().arg("FOO=bar").arg("ABC=xyz").run();
+    let out = new_ucmd!().arg("FOO=bar").arg("ABC=xyz").succeeds();
 
     assert_eq!(
         out.stdout_str()
@@ -299,7 +367,7 @@ fn test_empty_name() {
     new_ucmd!()
         .arg("-i")
         .arg("=xyz")
-        .run()
+        .succeeds()
         .stderr_only("env: warning: no name specified for value 'xyz'\n");
 }
 
@@ -458,7 +526,7 @@ fn test_split_string_into_args_s_escaped_c_not_allowed() {
     let out = scene.ucmd().args(&[r#"-S"\c""#]).fails().stderr_move_str();
     assert_eq!(
         out,
-        "env: '\\c' must not appear in double-quoted -S string\n"
+        "env: '\\c' must not appear in double-quoted -S string at position 2\n"
     );
 }
 
@@ -515,15 +583,21 @@ fn test_split_string_into_args_debug_output_whitespace_handling() {
 fn test_gnu_e20() {
     let scene = TestScenario::new(util_name!());
 
-    let env_bin = String::from(crate::common::util::TESTS_BINARY) + " " + util_name!();
+    let env_bin = String::from(uutests::util::get_tests_binary()) + " " + util_name!();
+    let input = [
+        String::from("-i"),
+        String::from(r#"-SA="B\_C=D" "#) + env_bin.escape_default().to_string().as_str() + "",
+    ];
 
-    let (input, output) = (
-        [
-            String::from("-i"),
-            String::from(r#"-SA="B\_C=D" "#) + env_bin.escape_default().to_string().as_str() + "",
-        ],
-        "A=B C=D\n",
-    );
+    let mut output = "A=B C=D\n".to_string();
+
+    // Workaround for the test to pass when coverage is being run.
+    // If enabled, the binary called by env_bin will most probably be
+    // instrumented for coverage, and thus will set the
+    // __LLVM_PROFILE_RT_INIT_ONCE
+    if env::var("__LLVM_PROFILE_RT_INIT_ONCE").is_ok() {
+        output.push_str("__LLVM_PROFILE_RT_INIT_ONCE=__LLVM_PROFILE_RT_INIT_ONCE\n");
+    }
 
     let out = scene.ucmd().args(&input).succeeds();
     assert_eq!(out.stdout_str(), output);
@@ -538,91 +612,91 @@ fn test_env_parsing_errors() {
         .arg("-S\\|echo hallo") // no quotes, invalid escape sequence |
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\|' in -S\n");
+        .stderr_is("env: invalid sequence '\\|' in -S at position 1\n");
 
     ts.ucmd()
         .arg("-S\\a") // no quotes, invalid escape sequence a
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\a' in -S\n");
+        .stderr_is("env: invalid sequence '\\a' in -S at position 1\n");
 
     ts.ucmd()
         .arg("-S\"\\a\"") // double quotes, invalid escape sequence a
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\a' in -S\n");
+        .stderr_is("env: invalid sequence '\\a' in -S at position 2\n");
 
     ts.ucmd()
         .arg(r#"-S"\a""#) // same as before, just using r#""#
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\a' in -S\n");
+        .stderr_is("env: invalid sequence '\\a' in -S at position 2\n");
 
     ts.ucmd()
         .arg("-S'\\a'") // single quotes, invalid escape sequence a
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\a' in -S\n");
+        .stderr_is("env: invalid sequence '\\a' in -S at position 2\n");
 
     ts.ucmd()
         .arg(r"-S\|\&\;") // no quotes, invalid escape sequence |
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\|' in -S\n");
+        .stderr_is("env: invalid sequence '\\|' in -S at position 1\n");
 
     ts.ucmd()
         .arg(r"-S\<\&\;") // no quotes, invalid escape sequence <
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\<' in -S\n");
+        .stderr_is("env: invalid sequence '\\<' in -S at position 1\n");
 
     ts.ucmd()
         .arg(r"-S\>\&\;") // no quotes, invalid escape sequence >
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\>' in -S\n");
+        .stderr_is("env: invalid sequence '\\>' in -S at position 1\n");
 
     ts.ucmd()
         .arg(r"-S\`\&\;") // no quotes, invalid escape sequence `
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\`' in -S\n");
+        .stderr_is("env: invalid sequence '\\`' in -S at position 1\n");
 
     ts.ucmd()
         .arg(r#"-S"\`\&\;""#) // double quotes, invalid escape sequence `
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\`' in -S\n");
+        .stderr_is("env: invalid sequence '\\`' in -S at position 2\n");
 
     ts.ucmd()
         .arg(r"-S'\`\&\;'") // single quotes, invalid escape sequence `
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\`' in -S\n");
+        .stderr_is("env: invalid sequence '\\`' in -S at position 2\n");
 
     ts.ucmd()
         .arg(r"-S\`") // ` escaped without quotes
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\`' in -S\n");
+        .stderr_is("env: invalid sequence '\\`' in -S at position 1\n");
 
     ts.ucmd()
         .arg(r#"-S"\`""#) // ` escaped in double quotes
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\`' in -S\n");
+        .stderr_is("env: invalid sequence '\\`' in -S at position 2\n");
 
     ts.ucmd()
         .arg(r"-S'\`'") // ` escaped in single quotes
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\`' in -S\n");
+        .stderr_is("env: invalid sequence '\\`' in -S at position 2\n");
 
     ts.ucmd()
         .args(&[r"-S\🦉"]) // ` escaped in single quotes
         .fails_with_code(125)
         .no_stdout()
-        .stderr_is("env: invalid sequence '\\\u{FFFD}' in -S\n"); // gnu doesn't show the owl. Instead a invalid unicode ?
+        .stderr_is("env: invalid sequence '\\\u{FFFD}' in -S at position 1\n"); // gnu doesn't show the owl. Instead a invalid unicode ?
 }
 
 #[test]
@@ -866,6 +940,89 @@ fn test_env_arg_ignore_signal_empty() {
         .stdout_contains("hello");
 }
 
+#[test]
+#[cfg(unix)]
+fn test_env_arg_ignore_signal_all_signals() {
+    let mut target = Target::new(&[]);
+    target.send_signal(Signal::SIGINT);
+    assert!(target.is_alive());
+}
+
+#[test]
+#[cfg(unix)]
+fn test_env_default_signal_pipe() {
+    let ts = TestScenario::new(util_name!());
+    run_sigpipe_script(&ts, &["--default-signal=PIPE"]);
+}
+
+#[test]
+#[cfg(unix)]
+fn test_env_default_signal_all_signals() {
+    let ts = TestScenario::new(util_name!());
+    run_sigpipe_script(&ts, &["--default-signal"]);
+}
+
+#[test]
+#[cfg(unix)]
+fn test_env_block_signal_flag() {
+    new_ucmd!()
+        .env("PATH", PATH)
+        .args(&["--block-signal", "true"])
+        .succeeds()
+        .no_stderr();
+}
+
+#[test]
+#[cfg(unix)]
+fn test_env_list_signal_handling_reports_ignore() {
+    let result = new_ucmd!()
+        .env("PATH", PATH)
+        .args(&["--ignore-signal=INT", "--list-signal-handling", "true"])
+        .succeeds();
+    let stderr = result.stderr_str();
+    assert!(
+        stderr.contains("INT") && stderr.contains("IGNORE"),
+        "unexpected signal listing: {stderr}"
+    );
+}
+
+#[cfg(unix)]
+fn run_sigpipe_script(ts: &TestScenario, extra_args: &[&str]) {
+    let shell = env::var("SHELL").unwrap_or_else(|_| String::from("sh"));
+    let _guard = SigpipeGuard::new();
+    let mut cmd = ts.ucmd();
+    cmd.env("PATH", PATH);
+    cmd.args(extra_args);
+    cmd.arg(shell);
+    cmd.arg("-c");
+    cmd.arg("trap - PIPE; seq 999999 2>err | head -n1 > out");
+    cmd.succeeds();
+    assert_eq!(ts.fixtures.read("out"), "1\n");
+    assert_eq!(ts.fixtures.read("err"), "");
+}
+
+#[cfg(unix)]
+struct SigpipeGuard {
+    previous: libc::sighandler_t,
+}
+
+#[cfg(unix)]
+impl SigpipeGuard {
+    fn new() -> Self {
+        let previous = unsafe { libc::signal(libc::SIGPIPE, libc::SIG_IGN) };
+        Self { previous }
+    }
+}
+
+#[cfg(unix)]
+impl Drop for SigpipeGuard {
+    fn drop(&mut self) {
+        unsafe {
+            libc::signal(libc::SIGPIPE, self.previous);
+        }
+    }
+}
+
 #[test]
 fn disallow_equals_sign_on_short_unset_option() {
     let ts = TestScenario::new(util_name!());
@@ -939,7 +1096,7 @@ mod tests_split_iterator {
     ///
     /// It tries to avoid introducing any unnecessary quotes or escape characters,
     /// but specifics regarding quoting style are left unspecified.
-    pub fn quote(s: &str) -> std::borrow::Cow<str> {
+    pub fn quote(s: &str) -> std::borrow::Cow<'_, str> {
         // We are going somewhat out of the way to provide
         // minimal amount of quoting in typical cases.
         match escape_style(s) {
@@ -1008,10 +1165,12 @@ mod tests_split_iterator {
 
     use std::ffi::OsString;
 
-    use env::native_int_str::{from_native_int_representation_owned, Convert, NCvt};
-    use env::parse_error::ParseError;
+    use env::{
+        EnvError,
+        native_int_str::{Convert, NCvt, from_native_int_representation_owned},
+    };
 
-    fn split(input: &str) -> Result<Vec<OsString>, ParseError> {
+    fn split(input: &str) -> Result<Vec<OsString>, EnvError> {
         ::env::split_iterator::split(&NCvt::convert(input)).map(|vec| {
             vec.into_iter()
                 .map(from_native_int_representation_owned)
@@ -1028,8 +1187,9 @@ mod tests_split_iterator {
                     );
                 }
                 Ok(actual) => {
-                    assert!(
-                        expected == actual.as_slice(),
+                    assert_eq!(
+                        expected,
+                        actual.as_slice(),
                         "[{i}] After split({input:?}).unwrap()\nexpected: {expected:?}\n  actual: {actual:?}\n"
                     );
                 }
@@ -1118,24 +1278,24 @@ mod tests_split_iterator {
     fn split_trailing_backslash() {
         assert_eq!(
             split("\\"),
-            Err(ParseError::InvalidBackslashAtEndOfStringInMinusS {
-                pos: 1,
-                quoting: "Delimiter".into()
-            })
+            Err(EnvError::EnvInvalidBackslashAtEndOfStringInMinusS(
+                1,
+                "Delimiter".into()
+            ))
         );
         assert_eq!(
             split(" \\"),
-            Err(ParseError::InvalidBackslashAtEndOfStringInMinusS {
-                pos: 2,
-                quoting: "Delimiter".into()
-            })
+            Err(EnvError::EnvInvalidBackslashAtEndOfStringInMinusS(
+                2,
+                "Delimiter".into()
+            ))
         );
         assert_eq!(
             split("a\\"),
-            Err(ParseError::InvalidBackslashAtEndOfStringInMinusS {
-                pos: 2,
-                quoting: "Unquoted".into()
-            })
+            Err(EnvError::EnvInvalidBackslashAtEndOfStringInMinusS(
+                2,
+                "Unquoted".into()
+            ))
         );
     }
 
@@ -1143,26 +1303,14 @@ mod tests_split_iterator {
     fn split_errors() {
         assert_eq!(
             split("'abc"),
-            Err(ParseError::MissingClosingQuote { pos: 4, c: '\'' })
-        );
-        assert_eq!(
-            split("\""),
-            Err(ParseError::MissingClosingQuote { pos: 1, c: '"' })
-        );
-        assert_eq!(
-            split("'\\"),
-            Err(ParseError::MissingClosingQuote { pos: 2, c: '\'' })
-        );
-        assert_eq!(
-            split("'\\"),
-            Err(ParseError::MissingClosingQuote { pos: 2, c: '\'' })
+            Err(EnvError::EnvMissingClosingQuote(4, '\''))
         );
+        assert_eq!(split("\""), Err(EnvError::EnvMissingClosingQuote(1, '"')));
+        assert_eq!(split("'\\"), Err(EnvError::EnvMissingClosingQuote(2, '\'')));
+        assert_eq!(split("'\\"), Err(EnvError::EnvMissingClosingQuote(2, '\'')));
         assert_eq!(
             split(r#""$""#),
-            Err(ParseError::ParsingOfVariableNameFailed {
-                pos: 2,
-                msg: "Missing variable name".into()
-            }),
+            Err(EnvError::EnvParsingOfMissingVariable(2)),
         );
     }
 
@@ -1170,26 +1318,25 @@ mod tests_split_iterator {
     fn split_error_fail_with_unknown_escape_sequences() {
         assert_eq!(
             split("\\a"),
-            Err(ParseError::InvalidSequenceBackslashXInMinusS { pos: 1, c: 'a' })
+            Err(EnvError::EnvInvalidSequenceBackslashXInMinusS(1, 'a'))
         );
         assert_eq!(
             split("\"\\a\""),
-            Err(ParseError::InvalidSequenceBackslashXInMinusS { pos: 2, c: 'a' })
+            Err(EnvError::EnvInvalidSequenceBackslashXInMinusS(2, 'a'))
         );
         assert_eq!(
             split("'\\a'"),
-            Err(ParseError::InvalidSequenceBackslashXInMinusS { pos: 2, c: 'a' })
+            Err(EnvError::EnvInvalidSequenceBackslashXInMinusS(2, 'a'))
         );
         assert_eq!(
             split(r#""\a""#),
-            Err(ParseError::InvalidSequenceBackslashXInMinusS { pos: 2, c: 'a' })
+            Err(EnvError::EnvInvalidSequenceBackslashXInMinusS(2, 'a'))
         );
         assert_eq!(
             split(r"\🦉"),
-            Err(ParseError::InvalidSequenceBackslashXInMinusS {
-                pos: 1,
-                c: '\u{FFFD}'
-            })
+            Err(EnvError::EnvInvalidSequenceBackslashXInMinusS(
+                1, '\u{FFFD}'
+            ))
         );
     }
 
@@ -1245,8 +1392,8 @@ mod test_raw_string_parser {
 
     use env::{
         native_int_str::{
-            from_native_int_representation, from_native_int_representation_owned,
-            to_native_int_representation, NativeStr,
+            NativeStr, from_native_int_representation, from_native_int_representation_owned,
+            to_native_int_representation,
         },
         string_expander::StringExpander,
         string_parser,
@@ -1511,3 +1658,307 @@ mod test_raw_string_parser {
         );
     }
 }
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_false() {
+    let scene = TestScenario::new("util");
+
+    let out = scene.ccmd("env").arg("sh").arg("is_a_tty.sh").succeeds();
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        "stdin is not a tty\nstdout is not a tty\nstderr is not a tty\n"
+    );
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stderr()),
+        "This is an error message.\n"
+    );
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_true() {
+    let scene = TestScenario::new("util");
+
+    let out = scene
+        .ccmd("env")
+        .arg("sh")
+        .arg("is_a_tty.sh")
+        .terminal_simulation(true)
+        .succeeds();
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        "stdin is a tty\r\nterminal size: 30 80\r\nstdout is a tty\r\nstderr is a tty\r\n"
+    );
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stderr()),
+        "This is an error message.\r\n"
+    );
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_for_stdin_only() {
+    let scene = TestScenario::new("util");
+
+    let out = scene
+        .ccmd("env")
+        .arg("sh")
+        .arg("is_a_tty.sh")
+        .terminal_sim_stdio(TerminalSimulation {
+            stdin: true,
+            stdout: false,
+            stderr: false,
+            ..Default::default()
+        })
+        .succeeds();
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        "stdin is a tty\nterminal size: 30 80\nstdout is not a tty\nstderr is not a tty\n"
+    );
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stderr()),
+        "This is an error message.\n"
+    );
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_for_stdout_only() {
+    let scene = TestScenario::new("util");
+
+    let out = scene
+        .ccmd("env")
+        .arg("sh")
+        .arg("is_a_tty.sh")
+        .terminal_sim_stdio(TerminalSimulation {
+            stdin: false,
+            stdout: true,
+            stderr: false,
+            ..Default::default()
+        })
+        .succeeds();
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        "stdin is not a tty\r\nstdout is a tty\r\nstderr is not a tty\r\n"
+    );
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stderr()),
+        "This is an error message.\n"
+    );
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_for_stderr_only() {
+    let scene = TestScenario::new("util");
+
+    let out = scene
+        .ccmd("env")
+        .arg("sh")
+        .arg("is_a_tty.sh")
+        .terminal_sim_stdio(TerminalSimulation {
+            stdin: false,
+            stdout: false,
+            stderr: true,
+            ..Default::default()
+        })
+        .succeeds();
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        "stdin is not a tty\nstdout is not a tty\nstderr is a tty\n"
+    );
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stderr()),
+        "This is an error message.\r\n"
+    );
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_size_information() {
+    let scene = TestScenario::new("util");
+
+    let out = scene
+        .ccmd("env")
+        .arg("sh")
+        .arg("is_a_tty.sh")
+        .terminal_sim_stdio(TerminalSimulation {
+            size: Some(libc::winsize {
+                ws_col: 40,
+                ws_row: 10,
+                ws_xpixel: 40 * 8,
+                ws_ypixel: 10 * 10,
+            }),
+            stdout: true,
+            stdin: true,
+            stderr: true,
+        })
+        .succeeds();
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        "stdin is a tty\r\nterminal size: 10 40\r\nstdout is a tty\r\nstderr is a tty\r\n"
+    );
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stderr()),
+        "This is an error message.\r\n"
+    );
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_pty_sends_eot_automatically() {
+    let scene = TestScenario::new("util");
+
+    let mut cmd = scene.ccmd("env");
+    cmd.timeout(std::time::Duration::from_secs(10));
+    cmd.args(&["cat", "-"]);
+    cmd.terminal_simulation(true);
+    let child = cmd.run_no_wait();
+    let out = child.wait().unwrap(); // cat would block if there is no eot
+
+    std::assert_eq!(String::from_utf8_lossy(out.stderr()), "");
+    std::assert_eq!(String::from_utf8_lossy(out.stdout()), "\r\n");
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_pty_pipes_into_data_and_sends_eot_automatically() {
+    let scene = TestScenario::new("util");
+
+    let message = "Hello stdin forwarding!";
+
+    let mut cmd = scene.ccmd("env");
+    cmd.args(&["cat", "-"]);
+    cmd.terminal_simulation(true);
+    cmd.pipe_in(message);
+    let child = cmd.run_no_wait();
+    let out = child.wait().unwrap();
+
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        format!("{message}\r\n")
+    );
+    std::assert_eq!(String::from_utf8_lossy(out.stderr()), "");
+}
+
+#[test]
+#[cfg(not(windows))]
+fn test_emoji_env_vars() {
+    new_ucmd!()
+        .arg("🎯_VAR=Hello 🌍")
+        .arg("printenv")
+        .arg("🎯_VAR")
+        .succeeds()
+        .stdout_contains("Hello 🌍");
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simulation_of_terminal_pty_write_in_data_and_sends_eot_automatically() {
+    let scene = TestScenario::new("util");
+
+    let mut cmd = scene.ccmd("env");
+    cmd.args(&["cat", "-"]);
+    cmd.terminal_simulation(true);
+    let mut child = cmd.run_no_wait();
+    child.write_in("Hello stdin forwarding via write_in!");
+    let out = child.wait().unwrap();
+
+    std::assert_eq!(
+        String::from_utf8_lossy(out.stdout()),
+        "Hello stdin forwarding via write_in!\r\n"
+    );
+    std::assert_eq!(String::from_utf8_lossy(out.stderr()), "");
+}
+
+#[test]
+fn test_env_french() {
+    new_ucmd!()
+        .arg("--verbo")
+        .env("LANG", "fr_FR")
+        .fails()
+        .stderr_contains("erreur : argument inattendu");
+}
+
+#[test]
+fn test_shebang_error() {
+    new_ucmd!()
+        .arg("\'-v \'")
+        .fails()
+        .stderr_contains("use -[v]S to pass options in shebang lines");
+}
+
+#[test]
+#[cfg(not(target_os = "windows"))]
+fn test_braced_variable_with_default_value() {
+    new_ucmd!()
+        .arg("-Secho ${UNSET_VAR_UNLIKELY_12345:fallback}")
+        .succeeds()
+        .stdout_is("fallback\n");
+}
+
+#[test]
+#[cfg(not(target_os = "windows"))]
+fn test_braced_variable_with_default_when_set() {
+    new_ucmd!()
+        .env("TEST_VAR_12345", "actual")
+        .arg("-Secho ${TEST_VAR_12345:fallback}")
+        .succeeds()
+        .stdout_is("actual\n");
+}
+
+#[test]
+#[cfg(not(target_os = "windows"))]
+fn test_simple_braced_variable() {
+    new_ucmd!()
+        .env("TEST_VAR_12345", "value")
+        .arg("-Secho ${TEST_VAR_12345}")
+        .succeeds()
+        .stdout_is("value\n");
+}
+
+#[test]
+fn test_braced_variable_error_missing_closing_brace() {
+    new_ucmd!()
+        .arg("-Secho ${FOO")
+        .fails_with_code(125)
+        .stderr_contains("Missing closing brace");
+}
+
+#[test]
+fn test_braced_variable_error_missing_closing_brace_after_default() {
+    new_ucmd!()
+        .arg("-Secho ${FOO:-value")
+        .fails_with_code(125)
+        .stderr_contains("Missing closing brace after default value");
+}
+
+#[test]
+fn test_braced_variable_error_starts_with_digit() {
+    new_ucmd!()
+        .arg("-Secho ${1FOO}")
+        .fails_with_code(125)
+        .stderr_contains("Unexpected character: '1'");
+}
+
+#[test]
+fn test_braced_variable_error_unexpected_character() {
+    new_ucmd!()
+        .arg("-Secho ${FOO?}")
+        .fails_with_code(125)
+        .stderr_contains("Unexpected character: '?'");
+}
+
+#[test]
+#[cfg(unix)]
+fn test_non_utf8_env_vars() {
+    use std::ffi::OsString;
+    use std::os::unix::ffi::OsStringExt;
+
+    let non_utf8_value = OsString::from_vec(b"hello\x80world".to_vec());
+    new_ucmd!()
+        .env("NON_UTF8_VAR", &non_utf8_value)
+        .succeeds()
+        .stdout_contains_bytes(b"NON_UTF8_VAR=hello\x80world");
+}
