@@ -3,8 +3,24 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 //
-// spell-checker:ignore binvalid finvalid hinvalid iinvalid linvalid nabcabc nabcabcabc ninvalid vinvalid winvalid dabc näää
-use crate::common::util::TestScenario;
+// spell-checker:ignore binvalid finvalid hinvalid iinvalid linvalid nabcabc nabcabcabc ninvalid vinvalid winvalid dabc näää févr
+use uutests::{at_and_ucmd, new_ucmd, util::TestScenario, util_name};
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_non_utf8_paths() {
+    use std::os::unix::ffi::OsStringExt;
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let filename = std::ffi::OsString::from_vec(vec![0xFF, 0xFE]);
+    std::fs::write(at.plus(&filename), b"line 1\nline 2\nline 3\n").unwrap();
+
+    ucmd.arg(&filename)
+        .succeeds()
+        .stdout_contains("1\t")
+        .stdout_contains("2\t")
+        .stdout_contains("3\t");
+}
 
 #[test]
 fn test_invalid_arg() {
@@ -15,7 +31,7 @@ fn test_invalid_arg() {
 fn test_stdin_no_newline() {
     new_ucmd!()
         .pipe_in("No Newline")
-        .run()
+        .succeeds()
         .stdout_is("     1\tNo Newline\n");
 }
 
@@ -24,7 +40,7 @@ fn test_stdin_newline() {
     new_ucmd!()
         .args(&["-s", "-", "-w", "1"])
         .pipe_in("Line One\nLine Two\n")
-        .run()
+        .succeeds()
         .stdout_is("1-Line One\n2-Line Two\n");
 }
 
@@ -32,7 +48,7 @@ fn test_stdin_newline() {
 fn test_padding_without_overflow() {
     new_ucmd!()
         .args(&["-i", "1000", "-s", "x", "-n", "rz", "simple.txt"])
-        .run()
+        .succeeds()
         .stdout_is(
             "000001xL1\n001001xL2\n002001xL3\n003001xL4\n004001xL5\n005001xL6\n006001xL7\n0070\
              01xL8\n008001xL9\n009001xL10\n010001xL11\n011001xL12\n012001xL13\n013001xL14\n014\
@@ -44,7 +60,7 @@ fn test_padding_without_overflow() {
 fn test_padding_with_overflow() {
     new_ucmd!()
         .args(&["-i", "1000", "-s", "x", "-n", "rz", "-w", "4", "simple.txt"])
-        .run()
+        .succeeds()
         .stdout_is(
             "0001xL1\n1001xL2\n2001xL3\n3001xL4\n4001xL5\n5001xL6\n6001xL7\n7001xL8\n8001xL9\n\
              9001xL10\n10001xL11\n11001xL12\n12001xL13\n13001xL14\n14001xL15\n",
@@ -73,7 +89,7 @@ fn test_sections_and_styles() {
             .args(&[
                 "-s", "|", "-n", "ln", "-w", "3", "-b", "a", "-l", "5", fixture,
             ])
-            .run()
+            .succeeds()
             .stdout_is(output);
     }
     // spell-checker:enable
@@ -190,6 +206,29 @@ fn test_number_separator() {
     }
 }
 
+#[test]
+#[cfg(target_os = "linux")]
+fn test_number_separator_non_utf8() {
+    use std::{ffi::OsString, os::unix::ffi::OsStringExt};
+
+    let separator_bytes = [0xFF, 0xFE];
+    let mut v = b"--number-separator=".to_vec();
+    v.extend_from_slice(&separator_bytes);
+
+    let arg = OsString::from_vec(v);
+
+    // Raw bytes should be preserved in the separator output
+    let mut expected = b"     1".to_vec();
+    expected.extend_from_slice(&separator_bytes);
+    expected.extend_from_slice(b"test\n");
+
+    new_ucmd!()
+        .arg(arg)
+        .pipe_in("test")
+        .succeeds()
+        .stdout_is_bytes(expected);
+}
+
 #[test]
 fn test_starting_line_number() {
     for arg in ["-v10", "--starting-line-number=10"] {
@@ -285,6 +324,25 @@ fn test_join_blank_lines() {
     }
 }
 
+#[test]
+fn test_join_blank_lines_zero() {
+    for arg in ["-l0", "--join-blank-lines=0"] {
+        new_ucmd!()
+            .arg(arg)
+            .arg("--body-numbering=a")
+            .pipe_in("\n\n\n\n\n\n")
+            .succeeds()
+            .stdout_is(concat!(
+                "     1\t\n",
+                "     2\t\n",
+                "     3\t\n",
+                "     4\t\n",
+                "     5\t\n",
+                "     6\t\n",
+            ));
+    }
+}
+
 #[test]
 fn test_join_blank_lines_multiple_files() {
     let scene = TestScenario::new(util_name!());
@@ -310,15 +368,6 @@ fn test_join_blank_lines_multiple_files() {
     }
 }
 
-#[test]
-fn test_join_blank_lines_zero() {
-    for arg in ["-l0", "--join-blank-lines=0"] {
-        new_ucmd!().arg(arg).fails().stderr_contains(
-            "Invalid line number of blank lines: ‘0’: Numerical result out of range",
-        );
-    }
-}
-
 #[test]
 fn test_invalid_join_blank_lines() {
     for arg in ["-linvalid", "--join-blank-lines=invalid"] {
@@ -579,7 +628,50 @@ fn test_section_delimiter() {
 }
 
 #[test]
-fn test_one_char_section_delimiter_expansion() {
+#[cfg(target_os = "linux")]
+fn test_section_delimiter_non_utf8() {
+    use std::{ffi::OsString, os::unix::ffi::OsStringExt};
+
+    fn create_arg(prefix: &[u8]) -> OsString {
+        let section_delimiter = [0xFF, 0xFE];
+        let mut v = prefix.to_vec();
+        v.extend_from_slice(&section_delimiter);
+        OsString::from_vec(v)
+    }
+
+    let short = create_arg(b"-d");
+    let long = create_arg(b"--section-delimiter=");
+
+    for arg in [short, long] {
+        let header_section: Vec<u8> =
+            vec![b'a', b'\n', 0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE, b'\n', b'b'];
+
+        new_ucmd!()
+            .arg(&arg)
+            .pipe_in(header_section)
+            .succeeds()
+            .stdout_is("     1\ta\n\n       b\n");
+
+        let body_section: Vec<u8> = vec![b'a', b'\n', 0xFF, 0xFE, 0xFF, 0xFE, b'\n', b'b'];
+
+        new_ucmd!()
+            .arg(&arg)
+            .pipe_in(body_section)
+            .succeeds()
+            .stdout_is("     1\ta\n\n     1\tb\n");
+
+        let footer_section: Vec<u8> = vec![b'a', b'\n', 0xFF, 0xFE, b'\n', b'b'];
+
+        new_ucmd!()
+            .arg(&arg)
+            .pipe_in(footer_section)
+            .succeeds()
+            .stdout_is("     1\ta\n\n       b\n");
+    }
+}
+
+#[test]
+fn test_one_char_section_delimiter() {
     for arg in ["-da", "--section-delimiter=a"] {
         new_ucmd!()
             .arg(arg)
@@ -601,6 +693,48 @@ fn test_one_char_section_delimiter_expansion() {
     }
 }
 
+#[test]
+#[cfg(target_os = "linux")]
+fn test_one_byte_section_delimiter() {
+    use std::{ffi::OsString, os::unix::ffi::OsStringExt};
+
+    fn create_arg(prefix: &[u8]) -> OsString {
+        let mut v = prefix.to_vec();
+        v.push(0xFF);
+        OsString::from_vec(v)
+    }
+
+    let short = create_arg(b"-d");
+    let long = create_arg(b"--section-delimiter=");
+
+    for arg in [short, long] {
+        let header_section: Vec<u8> =
+            vec![b'a', b'\n', 0xFF, b':', 0xFF, b':', 0xFF, b':', b'\n', b'b'];
+
+        new_ucmd!()
+            .arg(&arg)
+            .pipe_in(header_section)
+            .succeeds()
+            .stdout_is("     1\ta\n\n       b\n");
+
+        let body_section: Vec<u8> = vec![b'a', b'\n', 0xFF, b':', 0xFF, b':', b'\n', b'b'];
+
+        new_ucmd!()
+            .arg(&arg)
+            .pipe_in(body_section)
+            .succeeds()
+            .stdout_is("     1\ta\n\n     1\tb\n");
+
+        let footer_section: Vec<u8> = vec![b'a', b'\n', 0xFF, b':', b'\n', b'b'];
+
+        new_ucmd!()
+            .arg(&arg)
+            .pipe_in(footer_section)
+            .succeeds()
+            .stdout_is("     1\ta\n\n       b\n");
+    }
+}
+
 #[test]
 fn test_non_ascii_one_char_section_delimiter() {
     for arg in ["-dä", "--section-delimiter=ä"] {
@@ -651,3 +785,130 @@ fn test_directory_as_input() {
         .stderr_is(format!("nl: {dir}: Is a directory\n"))
         .stdout_contains(content);
 }
+
+#[test]
+fn test_file_with_non_utf8_content() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let filename = "file";
+    let content: &[u8] = b"a\n\xFF\xFE\nb";
+
+    at.write_bytes(filename, content);
+
+    // Raw bytes should be preserved in output (not converted to UTF-8 replacement chars)
+    let expected: Vec<u8> = b"     1\ta\n     2\t\xFF\xFE\n     3\tb\n".to_vec();
+    ucmd.arg(filename).succeeds().stdout_is_bytes(expected);
+}
+
+#[test]
+fn test_stdin_non_utf8_preserved() {
+    // Verify that non-UTF8 bytes are preserved in output, not converted to replacement chars
+    // This is important for locale compatibility
+    let input: Vec<u8> = b"f\xe9vr.\n".to_vec(); // "févr." in Latin-1
+    let expected: Vec<u8> = b"     1\tf\xe9vr.\n".to_vec();
+    new_ucmd!()
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is_bytes(expected);
+}
+
+// Regression tests for issue #9132: repeated flags should use last value
+#[test]
+fn test_repeated_body_numbering_flag() {
+    // -ba -bt should use -bt (t=nonempty)
+    new_ucmd!()
+        .args(&["-ba", "-bt"])
+        .pipe_in("a\n\nb\n\nc")
+        .succeeds()
+        .stdout_is("     1\ta\n       \n     2\tb\n       \n     3\tc\n");
+}
+
+#[test]
+fn test_repeated_header_numbering_flag() {
+    // -ha -ht should use -ht (number only nonempty lines in header)
+    new_ucmd!()
+        .args(&["-ha", "-ht"])
+        .pipe_in("\\:\\:\\:\na\nb\n\nc")
+        .succeeds()
+        .stdout_is("\n     1\ta\n     2\tb\n       \n     3\tc\n");
+}
+
+#[test]
+fn test_repeated_footer_numbering_flag() {
+    // -fa -ft should use -ft (t=nonempty in footer)
+    new_ucmd!()
+        .args(&["-fa", "-ft"])
+        .pipe_in("\\:\na\nb\n\nc")
+        .succeeds()
+        .stdout_is("\n     1\ta\n     2\tb\n       \n     3\tc\n");
+}
+
+#[test]
+fn test_repeated_number_format_flag() {
+    // -n ln -n rn should use -n rn (rn=right aligned)
+    new_ucmd!()
+        .args(&["-n", "ln", "-n", "rn"])
+        .pipe_in("a\nb\nc")
+        .succeeds()
+        .stdout_is("     1\ta\n     2\tb\n     3\tc\n");
+}
+
+#[test]
+fn test_repeated_number_separator_flag() {
+    // -s ':' -s '|' should use -s '|'
+    new_ucmd!()
+        .args(&["-s", ":", "-s", "|"])
+        .pipe_in("a\nb\nc")
+        .succeeds()
+        .stdout_is("     1|a\n     2|b\n     3|c\n");
+}
+
+#[test]
+fn test_repeated_number_width_flag() {
+    // -w 3 -w 8 should use -w 8
+    new_ucmd!()
+        .args(&["-w", "3", "-w", "8"])
+        .pipe_in("a\nb\nc")
+        .succeeds()
+        .stdout_is("       1\ta\n       2\tb\n       3\tc\n");
+}
+
+#[test]
+fn test_repeated_line_increment_flag() {
+    // -i 1 -i 5 should use -i 5
+    new_ucmd!()
+        .args(&["-i", "1", "-i", "5"])
+        .pipe_in("a\nb\nc")
+        .succeeds()
+        .stdout_is("     1\ta\n     6\tb\n    11\tc\n");
+}
+
+#[test]
+fn test_repeated_starting_line_number_flag() {
+    // -v 1 -v 10 should use -v 10
+    new_ucmd!()
+        .args(&["-v", "1", "-v", "10"])
+        .pipe_in("a\nb\nc")
+        .succeeds()
+        .stdout_is("    10\ta\n    11\tb\n    12\tc\n");
+}
+
+#[test]
+fn test_repeated_join_blank_lines_flag() {
+    // -l 1 -l 2 should use -l 2
+    new_ucmd!()
+        .args(&["-l", "1", "-l", "2", "-ba"])
+        .pipe_in("a\n\n\nb")
+        .succeeds()
+        .stdout_is("     1\ta\n       \n     2\t\n     3\tb\n");
+}
+
+#[test]
+fn test_repeated_section_delimiter_flag() {
+    // -d ':' -d '|' should use -d '|'
+    new_ucmd!()
+        .args(&["-d", ":", "-d", "|"])
+        .pipe_in("|:|:|:\na\nb\nc")
+        .succeeds()
+        .stdout_is("\n       a\n       b\n       c\n");
+}
