@@ -2,8 +2,26 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore axxbxx bxxaxx axxx axxxx xxaxx xxax xxxxa axyz zyax zyxa
-use crate::common::util::TestScenario;
+// spell-checker:ignore axxbxx bxxaxx axxx axxxx xxaxx xxax xxxxa axyz zyax zyxa bbaaa aaabc bcdddd cddddaaabc xyzabc abcxyzabc nbbaaa
+#[cfg(target_os = "linux")]
+use uutests::at_and_ucmd;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_tac_non_utf8_paths() {
+    use std::os::unix::ffi::OsStringExt;
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let filename = std::ffi::OsString::from_vec(vec![0xFF, 0xFE]);
+    std::fs::write(at.plus(&filename), b"line1\nline2\nline3\n").unwrap();
+
+    ucmd.arg(&filename)
+        .succeeds()
+        .stdout_is("line3\nline2\nline1\n");
+}
 
 #[test]
 fn test_invalid_arg() {
@@ -16,7 +34,7 @@ fn test_invalid_arg() {
 fn test_stdin_default() {
     new_ucmd!()
         .pipe_in("100\n200\n300\n400\n500")
-        .run()
+        .succeeds()
         .stdout_is("500400\n300\n200\n100\n");
 }
 
@@ -27,7 +45,7 @@ fn test_stdin_non_newline_separator() {
     new_ucmd!()
         .args(&["-s", ":"])
         .pipe_in("100:200:300:400:500")
-        .run()
+        .succeeds()
         .stdout_is("500400:300:200:100:");
 }
 
@@ -38,7 +56,7 @@ fn test_stdin_non_newline_separator_before() {
     new_ucmd!()
         .args(&["-b", "-s", ":"])
         .pipe_in("100:200:300:400:500")
-        .run()
+        .succeeds()
         .stdout_is(":500:400:300:200100");
 }
 
@@ -46,7 +64,7 @@ fn test_stdin_non_newline_separator_before() {
 fn test_single_default() {
     new_ucmd!()
         .arg("prime_per_line.txt")
-        .run()
+        .succeeds()
         .stdout_is_fixture("prime_per_line.expected");
 }
 
@@ -54,7 +72,7 @@ fn test_single_default() {
 fn test_single_non_newline_separator() {
     new_ucmd!()
         .args(&["-s", ":", "delimited_primes.txt"])
-        .run()
+        .succeeds()
         .stdout_is_fixture("delimited_primes.expected");
 }
 
@@ -62,7 +80,7 @@ fn test_single_non_newline_separator() {
 fn test_single_non_newline_separator_before() {
     new_ucmd!()
         .args(&["-b", "-s", ":", "delimited_primes.txt"])
-        .run()
+        .succeeds()
         .stdout_is_fixture("delimited_primes_before.expected");
 }
 
@@ -82,7 +100,7 @@ fn test_invalid_input() {
         .ucmd()
         .arg("a")
         .fails()
-        .stderr_contains("a: read error: Invalid argument");
+        .stderr_contains("a: read error: Is a directory");
 }
 
 #[test]
@@ -307,3 +325,94 @@ fn test_regex_before() {
         //          |--------||----||---|
         .stdout_is("+---+c+d-e+--++b+-+a+");
 }
+
+#[cfg(target_os = "linux")]
+#[test]
+fn test_failed_write_is_reported() {
+    new_ucmd!()
+        .pipe_in("hello")
+        .set_stdout(std::fs::File::create("/dev/full").unwrap())
+        .fails()
+        .stderr_is("tac: failed to write to stdout: No space left on device (os error 28)\n");
+}
+
+#[cfg(target_os = "linux")]
+#[test]
+fn test_stdin_bad_tmpdir_fallback() {
+    // When TMPDIR is invalid, tac falls back to reading stdin directly into memory
+    new_ucmd!()
+        .env("TMPDIR", "/nonexistent/dir")
+        .arg("-")
+        .pipe_in("a\nb\nc\n")
+        .succeeds()
+        .stdout_is("c\nb\na\n");
+}
+
+#[test]
+fn test_regex_or_operator() {
+    new_ucmd!()
+        .args(&["-r", "-s", r"[^x]\|x"])
+        .pipe_in("abc")
+        .succeeds()
+        .stdout_is("cba");
+}
+
+#[test]
+fn test_unescaped_middle_anchor() {
+    new_ucmd!()
+        .args(&["-r", "-s", r"1^2"])
+        .pipe_in("111^222")
+        .succeeds()
+        .stdout_is("22111^2");
+
+    new_ucmd!()
+        .args(&["-r", "-s", r"a$b"])
+        .pipe_in("aaa$bbb")
+        .succeeds()
+        .stdout_is("bbaaa$b");
+}
+
+#[test]
+fn test_escaped_middle_anchor() {
+    new_ucmd!()
+        .args(&["-r", "-s", r"c\^b"])
+        .pipe_in("aaabc^bcdddd")
+        .succeeds()
+        .stdout_is("cddddaaabc^b");
+
+    new_ucmd!()
+        .args(&["-r", "-s", r"c\$b"])
+        .pipe_in("aaabc$bcdddd")
+        .succeeds()
+        .stdout_is("cddddaaabc$b");
+}
+
+#[test]
+fn test_regular_start_anchor() {
+    new_ucmd!()
+        .args(&["-r", "-s", r"^abc"])
+        .pipe_in("xyzabc123abc")
+        .succeeds()
+        .stdout_is("xyzabc123abc");
+
+    new_ucmd!()
+        .args(&["-r", "-s", r"^b"])
+        .pipe_in("aaa\nbbb\nccc\n")
+        .succeeds()
+        .stdout_is("bb\nccc\naaa\nb");
+}
+
+#[test]
+fn test_regular_end_anchor() {
+    new_ucmd!()
+        .args(&["-r", "-s", r"abc$"])
+        .pipe_in("123abcxyzabc")
+        .succeeds()
+        .stdout_is("123abcxyzabc");
+
+    new_ucmd!()
+        .args(&["-r", "-s", r"b$"])
+        .pipe_in("aaa\nbbb\nccc\n")
+        .succeeds()
+        .stdout_is("\nccc\nbbaaa\nb");
+}
