@@ -4,10 +4,11 @@
 // file that was distributed with this source code.
 #![allow(clippy::borrow_as_ptr)]
 
-use crate::common::util::TestScenario;
+use uutests::{at_and_ucmd, new_ucmd};
+
 use regex::Regex;
-#[cfg(target_os = "linux")]
-use std::fmt::Write;
+use std::process::Stdio;
+use std::time::Duration;
 
 // tests for basic tee functionality.
 // inspired by:
@@ -91,24 +92,27 @@ fn test_tee_append() {
 }
 
 #[test]
-#[cfg(target_os = "linux")]
-fn test_tee_no_more_writeable_1() {
-    // equals to 'tee /dev/full out2 <multi_read' call
+fn test_tee_multiple_append_flags() {
+    // Test for bug: https://bugs.launchpad.net/ubuntu/+source/rust-coreutils/+bug/2134578
+    // The command should accept multiple -a flags for different files
     let (at, mut ucmd) = at_and_ucmd!();
-    let content = (1..=10).fold(String::new(), |mut output, x| {
-        writeln!(output, "{x}").unwrap();
-        output
-    });
-    let file_out = "tee_file_out";
+    let content = "don't fail me now rust";
+    let file1 = "log1";
+    let file2 = "log2";
 
-    ucmd.arg("/dev/full")
-        .arg(file_out)
-        .pipe_in(&content[..])
-        .fails()
-        .stdout_contains(&content)
-        .stderr_contains("No space left on device");
+    // Pre-populate files with some content to verify append behavior
+    at.write(file1, "existing1\n");
+    at.write(file2, "existing2\n");
 
-    assert_eq!(at.read(file_out), content);
+    ucmd.args(&["-a", file1, "-a", file2])
+        .pipe_in(content)
+        .succeeds()
+        .stdout_is(content);
+
+    assert!(at.file_exists(file1));
+    assert!(at.file_exists(file2));
+    assert_eq!(at.read(file1), format!("existing1\n{content}"));
+    assert_eq!(at.read(file2), format!("existing2\n{content}"));
 }
 
 #[test]
@@ -133,39 +137,61 @@ fn test_readonly() {
 }
 
 #[test]
-#[cfg(target_os = "linux")]
-fn test_tee_no_more_writeable_2() {
-    // should be equals to 'tee out1 out2 >/dev/full <multi_read' call
-    // but currently there is no way to redirect stdout to /dev/full
-    // so this test is disabled
-    let (_at, mut ucmd) = at_and_ucmd!();
-    let _content = (1..=10).fold(String::new(), |mut output, x| {
-        let _ = writeln!(output, "{x}");
-        output
+fn test_tee_output_not_buffered() {
+    // POSIX says: The tee utility shall not buffer output
+
+    // If the output is buffered, the test will hang, so we run it in
+    // a separate thread to stop execution by timeout.
+    let handle = std::thread::spawn(move || {
+        let content = "a";
+        let file_out = "tee_file_out";
+
+        let (at, mut ucmd) = at_and_ucmd!();
+        let mut child = ucmd
+            .arg(file_out)
+            .set_stdin(Stdio::piped())
+            .set_stdout(Stdio::piped())
+            .run_no_wait();
+
+        // We write to the input pipe, but do not close it. If the output is
+        // buffered, reading from output pipe will hang indefinitely, as we
+        // will never write anything else to it.
+        child.write_in(content.as_bytes());
+
+        let out = String::from_utf8(child.stdout_exact_bytes(1)).unwrap();
+        assert_eq!(&out, content);
+
+        // Writing to a file may take a couple hundreds nanoseconds
+        child.delay(1);
+        assert_eq!(at.read(file_out), content);
     });
-    let file_out_a = "tee_file_out_a";
-    let file_out_b = "tee_file_out_b";
 
-    let _result = ucmd
-        .arg(file_out_a)
-        .arg(file_out_b)
-        .pipe_in("/dev/full")
-        .succeeds(); // TODO: expected to succeed currently; change to fails() when required
+    // Give some time for the `tee` to create an output file. Some platforms
+    // take a lot of time to spin up the process and create the output file
+    for _ in 0..100 {
+        std::thread::sleep(Duration::from_millis(1));
+        if handle.is_finished() {
+            break;
+        }
+    }
 
-    // TODO: comment in after https://github.com/uutils/coreutils/issues/1805 is fixed
-    // assert_eq!(at.read(file_out_a), content);
-    // assert_eq!(at.read(file_out_b), content);
-    // assert!(result.stderr.contains("No space left on device"));
+    assert!(
+        handle.is_finished(),
+        "Nothing was received through output pipe"
+    );
+    handle.join().unwrap();
 }
 
 #[cfg(target_os = "linux")]
 mod linux_only {
-    use crate::common::util::{AtPath, CmdResult, TestScenario, UCommand};
+    use uutests::util::{AtPath, CmdResult, UCommand};
 
     use std::fmt::Write;
     use std::fs::File;
     use std::process::Stdio;
     use std::time::Duration;
+    use uutests::at_and_ucmd;
+    use uutests::new_ucmd;
 
     fn make_broken_pipe() -> File {
         use libc::c_int;
@@ -238,8 +264,7 @@ mod linux_only {
         );
         assert!(
             result.stderr_str().contains(message),
-            "Expected to see error message fragment {} in stderr, but did not.\n stderr = {}",
-            message,
+            "Expected to see error message fragment {message} in stderr, but did not.\n stderr = {}",
             std::str::from_utf8(result.stderr()).unwrap(),
         );
     }
@@ -269,13 +294,14 @@ mod linux_only {
         let compare = at.read(name);
         assert!(
             compare.len() < contents.len(),
-            "Too many bytes ({}) written to {} (should be a short count from {})",
+            "Too many bytes ({}) written to {name} (should be a short count from {})",
             compare.len(),
-            name,
             contents.len()
         );
-        assert!(contents.starts_with(&compare),
-                "Expected truncated output to be a prefix of the correct output, but it isn't.\n Correct: {contents}\n Compare: {compare}");
+        assert!(
+            contents.starts_with(&compare),
+            "Expected truncated output to be a prefix of the correct output, but it isn't.\n Correct: {contents}\n Compare: {compare}"
+        );
     }
 
     #[test]
@@ -576,4 +602,138 @@ mod linux_only {
         expect_success(&output);
         expect_correct(file_out_a, &at, content.as_str());
     }
+
+    #[test]
+    fn test_tee_no_more_writeable_1() {
+        // equals to 'tee /dev/full out2 <multi_read' call
+        let (at, mut ucmd) = at_and_ucmd!();
+        let content = (1..=10).fold(String::new(), |mut output, x| {
+            writeln!(output, "{x}").unwrap();
+            output
+        });
+        let file_out = "tee_file_out";
+
+        ucmd.arg("/dev/full")
+            .arg(file_out)
+            .pipe_in(&content[..])
+            .fails()
+            .stdout_contains(&content)
+            .stderr_contains("No space left on device");
+
+        assert_eq!(at.read(file_out), content);
+    }
+
+    #[test]
+    fn test_tee_no_more_writeable_2() {
+        use std::fs::File;
+        let (at, mut ucmd) = at_and_ucmd!();
+        let content = (1..=10).fold(String::new(), |mut output, x| {
+            let _ = writeln!(output, "{x}");
+            output
+        });
+        let file_out_a = "tee_file_out_a";
+        let file_out_b = "tee_file_out_b";
+        let dev_full = File::options().append(true).open("/dev/full").unwrap();
+
+        let result = ucmd
+            .arg(file_out_a)
+            .arg(file_out_b)
+            .set_stdout(dev_full)
+            .pipe_in(content.as_bytes())
+            .fails();
+
+        assert_eq!(at.read(file_out_a), content);
+        assert_eq!(at.read(file_out_b), content);
+        assert!(result.stderr_str().contains("No space left on device"));
+    }
+}
+
+// Additional cross-platform tee tests to cover GNU compatibility around --output-error
+#[test]
+fn test_output_error_flag_without_value_defaults_warn_nopipe() {
+    // When --output-error is present without an explicit value, it should default to warn-nopipe
+    // We can't easily simulate a broken pipe across all platforms here, but we can ensure
+    // the flag is accepted without error and basic tee functionality still works.
+    let (at, mut ucmd) = at_and_ucmd!();
+    let file_out = "tee_output_error_default.txt";
+    let content = "abc";
+
+    let result = ucmd
+        .arg("--output-error")
+        .arg(file_out)
+        .pipe_in(content)
+        .succeeds();
+
+    result.stdout_is(content);
+    assert!(at.file_exists(file_out));
+    assert_eq!(at.read(file_out), content);
+}
+// Unix-only: presence-only --output-error should not crash on broken pipe.
+// Current implementation may exit zero; we only assert the process exits to avoid flakiness.
+// TODO: When semantics are aligned with GNU warn-nopipe, strengthen assertions here.
+#[cfg(all(unix, not(target_os = "freebsd")))]
+#[test]
+fn test_output_error_presence_only_broken_pipe_unix() {
+    use std::fs::File;
+    use std::os::unix::io::FromRawFd;
+
+    unsafe {
+        let mut fds: [libc::c_int; 2] = [0, 0];
+        assert_eq!(libc::pipe(fds.as_mut_ptr()), 0, "Failed to create pipe");
+        // Close the read end to simulate a broken pipe on stdout
+        let _read_end = File::from_raw_fd(fds[0]);
+        let write_end = File::from_raw_fd(fds[1]);
+
+        let content = (0..10_000).map(|_| "x").collect::<String>();
+        let result = new_ucmd!()
+            .arg("--output-error") // presence-only flag
+            .set_stdout(write_end)
+            .pipe_in(content.as_bytes())
+            .run();
+
+        // Assert that a status was produced (i.e., process exited) and no crash occurred.
+        assert!(result.try_exit_status().is_some(), "process did not exit");
+    }
+}
+
+// Skip on FreeBSD due to repeated CI hangs in FreeBSD VM (see PR #8684)
+#[cfg(all(unix, not(target_os = "freebsd")))]
+#[test]
+fn test_broken_pipe_early_termination_stdout_only() {
+    use std::fs::File;
+    use std::os::unix::io::FromRawFd;
+
+    // Create a broken stdout by creating a pipe and dropping the read end
+    unsafe {
+        let mut fds: [libc::c_int; 2] = [0, 0];
+        assert_eq!(libc::pipe(fds.as_mut_ptr()), 0, "Failed to create pipe");
+        // Close the read end immediately to simulate a broken pipe
+        let _read_end = File::from_raw_fd(fds[0]);
+        let write_end = File::from_raw_fd(fds[1]);
+
+        let content = (0..10_000).map(|_| "x").collect::<String>();
+        let mut proc = new_ucmd!();
+        let result = proc
+            .set_stdout(write_end)
+            .ignore_stdin_write_error()
+            .pipe_in(content.as_bytes())
+            .run();
+
+        // GNU tee exits nonzero on broken pipe unless configured otherwise; implementation
+        // details vary by mode, but we should not panic and should return an exit status.
+        // Assert that a status was produced (i.e., process exited) and no crash occurred.
+        assert!(result.try_exit_status().is_some(), "process did not exit");
+    }
+}
+
+#[test]
+fn test_write_failure_reports_error_and_nonzero_exit() {
+    // Simulate a file open failure which should be reported via show_error and cause a failure
+    let (at, mut ucmd) = at_and_ucmd!();
+    // Create a directory and try to use it as an output file (open will fail)
+    at.mkdir("out_dir");
+
+    let result = ucmd.arg("out_dir").pipe_in("data").fails();
+
+    assert!(!result.stderr_str().is_empty());
 }
