@@ -2,14 +2,18 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore (formats) cymdhm cymdhms mdhm mdhms ymdhm ymdhms datetime mktime
+// spell-checker:ignore (formats) cymdhm cymdhms datetime mdhm mdhms mktime strtime ymdhm ymdhms
 
-use crate::common::util::{AtPath, TestScenario};
+use filetime::FileTime;
 #[cfg(not(target_os = "freebsd"))]
 use filetime::set_symlink_file_times;
-use filetime::FileTime;
+use jiff::{fmt::strtime, tz::TimeZone};
 use std::fs::remove_file;
 use std::path::PathBuf;
+use uutests::at_and_ucmd;
+use uutests::new_ucmd;
+use uutests::util::{AtPath, TestScenario};
+use uutests::util_name;
 
 fn get_file_times(at: &AtPath, path: &str) -> (FileTime, FileTime) {
     let m = at.metadata(path);
@@ -33,11 +37,10 @@ fn set_file_times(at: &AtPath, path: &str, atime: FileTime, mtime: FileTime) {
 }
 
 fn str_to_filetime(format: &str, s: &str) -> FileTime {
-    let tm = chrono::NaiveDateTime::parse_from_str(s, format).unwrap();
-    FileTime::from_unix_time(
-        tm.and_utc().timestamp(),
-        tm.and_utc().timestamp_subsec_nanos(),
-    )
+    let tm = strtime::parse(format, s).unwrap();
+    let dt = tm.to_datetime().unwrap();
+    let ts = dt.to_zoned(TimeZone::UTC).unwrap().timestamp();
+    FileTime::from_unix_time(ts.as_second(), ts.subsec_nanosecond() as u32)
 }
 
 #[test]
@@ -118,6 +121,69 @@ fn test_touch_set_mdhms_time() {
     assert_eq!(mtime.unix_seconds() - start_of_year.unix_seconds(), 45296);
 }
 
+#[test]
+#[cfg(target_pointer_width = "64")]
+fn test_touch_2_digit_years_68() {
+    // 68 and before are 20xx
+    // it will fail on 32 bits, because of wraparound for anything after
+    // 2038-01-19
+    let (at, mut ucmd) = at_and_ucmd!();
+    let file = "test_touch_set_two_digit_68_time";
+
+    ucmd.args(&["-t", "6801010000", file])
+        .succeeds()
+        .no_output();
+
+    assert!(at.file_exists(file));
+
+    //  January 1, 2068, 00:00:00
+    let expected = FileTime::from_unix_time(3_092_601_600, 0);
+    let (atime, mtime) = get_file_times(&at, file);
+    assert_eq!(atime, mtime);
+    assert_eq!(atime, expected);
+    assert_eq!(mtime, expected);
+}
+
+#[test]
+fn test_touch_2_digit_years_2038() {
+    // Same as test_touch_2_digit_years_68 but for 32 bits systems
+    // we test a date before the y2038 bug
+    let (at, mut ucmd) = at_and_ucmd!();
+    let file = "test_touch_set_two_digit_68_time";
+
+    ucmd.args(&["-t", "3801010000", file])
+        .succeeds()
+        .no_output();
+
+    assert!(at.file_exists(file));
+
+    // January 1, 2038, 00:00:00
+    let expected = FileTime::from_unix_time(2_145_916_800, 0);
+    let (atime, mtime) = get_file_times(&at, file);
+    assert_eq!(atime, mtime);
+    assert_eq!(atime, expected);
+    assert_eq!(mtime, expected);
+}
+
+#[test]
+fn test_touch_2_digit_years_69() {
+    // 69 and after are 19xx
+    let (at, mut ucmd) = at_and_ucmd!();
+    let file = "test_touch_set_two_digit_69_time";
+
+    ucmd.args(&["-t", "6901010000", file])
+        .succeeds()
+        .no_output();
+
+    assert!(at.file_exists(file));
+    // January 1, 1969, 00:00:00
+    let expected = FileTime::from_unix_time(-31_536_000, 0);
+    let (atime, mtime) = get_file_times(&at, file);
+    assert_eq!(atime, mtime);
+    assert_eq!(atime, expected);
+    assert_eq!(mtime, expected);
+}
+
 #[test]
 fn test_touch_set_ymdhm_time() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -213,17 +279,18 @@ fn test_touch_set_only_atime() {
 
         let start_of_year = str_to_filetime("%Y%m%d%H%M", "201501010000");
         let (atime, mtime) = get_file_times(&at, file);
-        assert!(atime != mtime);
+        assert_ne!(atime, mtime);
         assert_eq!(atime.unix_seconds() - start_of_year.unix_seconds(), 45240);
     }
 }
 
 #[test]
 fn test_touch_set_only_mtime_failed() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "test_touch_set_only_mtime";
 
-    ucmd.args(&["-t", "2015010112342", "-m", file]).fails();
+    new_ucmd!()
+        .args(&["-t", "2015010112342", "-m", file])
+        .fails();
 }
 
 #[test]
@@ -285,17 +352,17 @@ fn test_touch_set_both_offset_date_and_reference() {
 
 #[test]
 fn test_touch_set_both_time_and_date() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "test_touch_set_both_time_and_date";
 
-    ucmd.args(&[
-        "-t",
-        "2015010112342",
-        "-d",
-        "Thu Jan 01 12:34:00 2015",
-        file,
-    ])
-    .fails();
+    new_ucmd!()
+        .args(&[
+            "-t",
+            "2015010112342",
+            "-d",
+            "Thu Jan 01 12:34:00 2015",
+            file,
+        ])
+        .fails();
 }
 
 #[test]
@@ -314,7 +381,7 @@ fn test_touch_set_only_mtime() {
 
         let start_of_year = str_to_filetime("%Y%m%d%H%M", "201501010000");
         let (atime, mtime) = get_file_times(&at, file);
-        assert!(atime != mtime);
+        assert_ne!(atime, mtime);
         assert_eq!(mtime.unix_seconds() - start_of_year.unix_seconds(), 45240);
     }
 }
@@ -370,7 +437,7 @@ fn test_touch_no_dereference() {
 
 #[test]
 fn test_touch_reference() {
-    let scenario = TestScenario::new("touch");
+    let scenario = TestScenario::new(util_name!());
     let (at, mut _ucmd) = (scenario.fixtures.clone(), scenario.ucmd());
     let file_a = "test_touch_reference_a";
     let file_b = "test_touch_reference_b";
@@ -396,6 +463,31 @@ fn test_touch_reference() {
     }
 }
 
+#[test]
+fn test_touch_reference_dangling() {
+    let temp_dir = tempfile::tempdir().unwrap();
+    let nonexistent_target = temp_dir.path().join("nonexistent_target");
+    let dangling_symlink = temp_dir.path().join("test_touch_reference_dangling");
+
+    #[cfg(not(windows))]
+    {
+        std::os::unix::fs::symlink(&nonexistent_target, &dangling_symlink).unwrap();
+    }
+    #[cfg(windows)]
+    {
+        std::os::windows::fs::symlink_file(&nonexistent_target, &dangling_symlink).unwrap();
+    }
+
+    new_ucmd!()
+        .args(&[
+            "--reference",
+            dangling_symlink.to_str().unwrap(),
+            "some_file",
+        ])
+        .fails()
+        .stderr_contains("touch: failed to get attributes of");
+}
+
 #[test]
 fn test_touch_set_date() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -618,7 +710,8 @@ fn test_touch_set_date_relative_smoke() {
         "2 seconds",
         "2 years 1 week",
         "2 days ago",
-        "2 months and 1 second",
+        "2 months 1 second",
+        "a",
     ];
     for time in times {
         let (at, mut ucmd) = at_and_ucmd!();
@@ -628,19 +721,14 @@ fn test_touch_set_date_relative_smoke() {
             .no_stderr()
             .no_stdout();
     }
-    let (at, mut ucmd) = at_and_ucmd!();
-    at.touch("f");
-    ucmd.args(&["-d", "a", "f"])
-        .fails()
-        .stderr_contains("touch: Unable to parse date");
 }
 
 #[test]
 fn test_touch_set_date_wrong_format() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "test_touch_set_date_wrong_format";
 
-    ucmd.args(&["-d", "2005-43-21", file])
+    new_ucmd!()
+        .args(&["-d", "2005-43-21", file])
         .fails()
         .stderr_contains("Unable to parse date: 2005-43-21");
 }
@@ -664,7 +752,6 @@ fn test_touch_mtime_dst_succeeds() {
 #[test]
 #[cfg(unix)]
 fn test_touch_mtime_dst_fails() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "test_touch_set_mtime_dst_fails";
 
     // Some timezones use daylight savings time, this leads to problems if the
@@ -673,7 +760,8 @@ fn test_touch_mtime_dst_fails() {
     // invalid.
     // See https://www.gnu.org/software/libc/manual/html_node/TZ-Variable.html
     // for information on the TZ variable, which where the string is copied from.
-    ucmd.env("TZ", "EST+5EDT,M3.2.0/2,M11.1.0/2")
+    new_ucmd!()
+        .env("TZ", "EST+5EDT,M3.2.0/2,M11.1.0/2")
         .args(&["-m", "-t", "202003080200", file])
         .fails();
 }
@@ -681,9 +769,9 @@ fn test_touch_mtime_dst_fails() {
 #[test]
 #[cfg(unix)]
 fn test_touch_system_fails() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "/";
-    ucmd.args(&[file])
+    new_ucmd!()
+        .args(&[file])
         .fails()
         .stderr_contains("setting times of '/'");
 }
@@ -691,9 +779,8 @@ fn test_touch_system_fails() {
 #[test]
 #[cfg(not(target_os = "windows"))]
 fn test_touch_trailing_slash() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "no-file/";
-    ucmd.args(&[file]).fails().stderr_only(format!(
+    new_ucmd!().args(&[file]).fails().stderr_only(format!(
         "touch: cannot touch '{file}': No such file or directory\n"
     ));
 }
@@ -701,11 +788,11 @@ fn test_touch_trailing_slash() {
 #[test]
 #[cfg(target_os = "windows")]
 fn test_touch_trailing_slash_windows() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "no-file/";
-    ucmd.args(&[file]).fails().stderr_only(format!(
-        "touch: cannot touch '{file}': The filename, directory name, or volume label syntax is incorrect.\n"
-    ));
+    new_ucmd!()
+        .args(&[file])
+        .fails()
+        .stderr_contains(format!("touch: cannot touch '{file}'"));
 }
 
 #[test]
@@ -739,7 +826,7 @@ fn test_touch_changes_time_of_file_in_stdout() {
         .no_stderr();
 
     let (_, mtime_after) = get_file_times(&at, file);
-    assert!(mtime_after != mtime);
+    assert_ne!(mtime_after, mtime);
 }
 
 #[test]
@@ -758,8 +845,7 @@ fn test_touch_permission_denied_error_msg() {
 
     let full_path = at.plus_as_string(path_str);
     ucmd.arg(&full_path).fails().stderr_only(format!(
-        "touch: cannot touch '{}': Permission denied\n",
-        &full_path
+        "touch: cannot touch '{full_path}': Permission denied\n",
     ));
 }
 
@@ -866,17 +952,15 @@ fn test_touch_no_dereference_dangling() {
 #[test]
 #[cfg(not(target_os = "openbsd"))]
 fn test_touch_dash() {
-    let (_, mut ucmd) = at_and_ucmd!();
-
-    ucmd.args(&["-h", "-"]).succeeds().no_stderr().no_stdout();
+    new_ucmd!().args(&["-h", "-"]).succeeds().no_output();
 }
 
 #[test]
 fn test_touch_invalid_date_format() {
-    let (_at, mut ucmd) = at_and_ucmd!();
     let file = "test_touch_invalid_date_format";
 
-    ucmd.args(&["-m", "-t", "+1000000000000 years", file])
+    new_ucmd!()
+        .args(&["-m", "-t", "+1000000000000 years", file])
         .fails()
         .stderr_contains("touch: invalid date format '+1000000000000 years'");
 }
@@ -941,3 +1025,39 @@ fn test_obsolete_posix_format_with_year() {
     assert!(at.file_exists("11111111"));
     assert!(!at.file_exists("0101000090"));
 }
+
+#[test]
+fn test_touch_f_option() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let file = "test_f_option.txt";
+
+    ucmd.args(&["-f", file]).succeeds().no_output();
+
+    assert!(at.file_exists(file));
+    at.remove(file);
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_touch_non_utf8_paths() {
+    use std::ffi::OsStr;
+    use std::os::unix::ffi::OsStrExt;
+
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let non_utf8_bytes = b"test_\xFF\xFE.txt";
+    let non_utf8_name = OsStr::from_bytes(non_utf8_bytes);
+
+    scene.ucmd().arg(non_utf8_name).succeeds().no_output();
+    assert!(std::fs::metadata(at.plus(non_utf8_name)).is_ok());
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_touch_device_files() {
+    let (_, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&["/dev/null", "/dev/zero", "/dev/full", "/dev/random"])
+        .succeeds()
+        .no_output();
+}
