@@ -3,9 +3,12 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-use crate::common::util::{vec_of_size, TestScenario};
+#[cfg(unix)]
+use uutests::at_and_ucmd;
+use uutests::new_ucmd;
+use uutests::util::vec_of_size;
 
-// spell-checker:ignore (flags) lwmcL clmwL ; (path) bogusfile emptyfile manyemptylines moby notrailingnewline onelongemptyline onelongword weirdchars
+// spell-checker:ignore (flags) lwmcL clmwL ; (path) bogusfile emptyfile manyemptylines moby notrailingnewline onelongemptyline onelongword weirdchars ioerrdir
 #[test]
 fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
@@ -43,7 +46,7 @@ fn test_count_bytes_large_stdin() {
 fn test_stdin_default() {
     new_ucmd!()
         .pipe_in_fixture("lorem_ipsum.txt")
-        .run()
+        .succeeds()
         .stdout_is("     13     109     772\n");
 }
 
@@ -52,7 +55,7 @@ fn test_stdin_explicit() {
     new_ucmd!()
         .pipe_in_fixture("lorem_ipsum.txt")
         .arg("-")
-        .run()
+        .succeeds()
         .stdout_is("     13     109     772 -\n");
 }
 
@@ -61,8 +64,8 @@ fn test_utf8() {
     new_ucmd!()
         .args(&["-lwmcL"])
         .pipe_in_fixture("UTF_8_test.txt")
-        .run()
-        .stdout_is("    303    2119   22457   23025      79\n");
+        .succeeds()
+        .stdout_is("    303    2178   22457   23025      79\n");
 }
 
 #[test]
@@ -70,7 +73,7 @@ fn test_utf8_words() {
     new_ucmd!()
         .arg("-w")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("89\n");
 }
 
@@ -79,7 +82,7 @@ fn test_utf8_line_length_words() {
     new_ucmd!()
         .arg("-Lw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     89      48\n");
 }
 
@@ -88,7 +91,7 @@ fn test_utf8_line_length_chars() {
     new_ucmd!()
         .arg("-Lm")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("    442      48\n");
 }
 
@@ -97,7 +100,7 @@ fn test_utf8_line_length_chars_words() {
     new_ucmd!()
         .arg("-Lmw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     89     442      48\n");
 }
 
@@ -106,7 +109,7 @@ fn test_utf8_chars() {
     new_ucmd!()
         .arg("-m")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("442\n");
 }
 
@@ -115,7 +118,7 @@ fn test_utf8_bytes_chars() {
     new_ucmd!()
         .arg("-cm")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("    442     513\n");
 }
 
@@ -124,7 +127,7 @@ fn test_utf8_bytes_lines() {
     new_ucmd!()
         .arg("-cl")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25     513\n");
 }
 
@@ -133,7 +136,7 @@ fn test_utf8_bytes_chars_lines() {
     new_ucmd!()
         .arg("-cml")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25     442     513\n");
 }
 
@@ -142,7 +145,7 @@ fn test_utf8_chars_words() {
     new_ucmd!()
         .arg("-mw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     89     442\n");
 }
 
@@ -151,7 +154,7 @@ fn test_utf8_line_length_lines() {
     new_ucmd!()
         .arg("-Ll")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25      48\n");
 }
 
@@ -160,7 +163,7 @@ fn test_utf8_line_length_lines_words() {
     new_ucmd!()
         .arg("-Llw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25      89      48\n");
 }
 
@@ -169,7 +172,7 @@ fn test_utf8_lines_chars() {
     new_ucmd!()
         .arg("-ml")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25     442\n");
 }
 
@@ -178,7 +181,7 @@ fn test_utf8_lines_words_chars() {
     new_ucmd!()
         .arg("-mlw")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25      89     442\n");
 }
 
@@ -187,7 +190,7 @@ fn test_utf8_line_length_lines_chars() {
     new_ucmd!()
         .arg("-Llm")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25     442      48\n");
 }
 
@@ -196,7 +199,7 @@ fn test_utf8_all() {
     new_ucmd!()
         .arg("-lwmcL")
         .pipe_in_fixture("UTF_8_weirdchars.txt")
-        .run()
+        .succeeds()
         .stdout_is("     25      89     442     513      48\n");
 }
 
@@ -206,7 +209,7 @@ fn test_ascii_control() {
     new_ucmd!()
         .arg("-w")
         .pipe_in(*b"\x01\n")
-        .run()
+        .succeeds()
         .stdout_is("1\n");
 }
 
@@ -215,7 +218,7 @@ fn test_stdin_line_len_regression() {
     new_ucmd!()
         .args(&["-L"])
         .pipe_in("\n123456")
-        .run()
+        .succeeds()
         .stdout_is("6\n");
 }
 
@@ -224,7 +227,7 @@ fn test_stdin_only_bytes() {
     new_ucmd!()
         .args(&["-c"])
         .pipe_in_fixture("lorem_ipsum.txt")
-        .run()
+        .succeeds()
         .stdout_is("772\n");
 }
 
@@ -233,7 +236,7 @@ fn test_stdin_all_counts() {
     new_ucmd!()
         .args(&["-c", "-m", "-l", "-L", "-w"])
         .pipe_in_fixture("alice_in_wonderland.txt")
-        .run()
+        .succeeds()
         .stdout_is("      5      57     302     302      66\n");
 }
 
@@ -241,7 +244,7 @@ fn test_stdin_all_counts() {
 fn test_single_default() {
     new_ucmd!()
         .arg("moby_dick.txt")
-        .run()
+        .succeeds()
         .stdout_is("  18  204 1115 moby_dick.txt\n");
 }
 
@@ -249,7 +252,7 @@ fn test_single_default() {
 fn test_single_only_lines() {
     new_ucmd!()
         .args(&["-l", "moby_dick.txt"])
-        .run()
+        .succeeds()
         .stdout_is("18 moby_dick.txt\n");
 }
 
@@ -257,7 +260,7 @@ fn test_single_only_lines() {
 fn test_single_only_bytes() {
     new_ucmd!()
         .args(&["-c", "lorem_ipsum.txt"])
-        .run()
+        .succeeds()
         .stdout_is("772 lorem_ipsum.txt\n");
 }
 
@@ -265,48 +268,45 @@ fn test_single_only_bytes() {
 fn test_single_all_counts() {
     new_ucmd!()
         .args(&["-c", "-l", "-L", "-m", "-w", "alice_in_wonderland.txt"])
-        .run()
+        .succeeds()
         .stdout_is("  5  57 302 302  66 alice_in_wonderland.txt\n");
 }
 
 #[cfg(unix)]
 #[test]
 fn test_gnu_compatible_quotation() {
-    let scene = TestScenario::new(util_name!());
-    let at = &scene.fixtures;
+    let (at, mut ucmd) = at_and_ucmd!();
+
     at.mkdir("some-dir1");
     at.touch("some-dir1/12\n34.txt");
-    scene
-        .ucmd()
-        .args(&["some-dir1/12\n34.txt"])
-        .run()
+
+    ucmd.args(&["some-dir1/12\n34.txt"])
+        .succeeds()
         .stdout_is("0 0 0 'some-dir1/12'$'\\n''34.txt'\n");
 }
 
 #[cfg(feature = "test_risky_names")]
 #[test]
 fn test_non_unicode_names() {
-    let scene = TestScenario::new(util_name!());
+    let (at, mut ucmd) = at_and_ucmd!();
+
     let target1 = uucore::os_str_from_bytes(b"some-dir1/1\xC0\n.txt")
         .expect("Only unix platforms can test non-unicode names");
     let target2 = uucore::os_str_from_bytes(b"some-dir1/2\xC0\t.txt")
         .expect("Only unix platforms can test non-unicode names");
-    let at = &scene.fixtures;
+
     at.mkdir("some-dir1");
     at.touch(&target1);
     at.touch(&target2);
-    scene
-        .ucmd()
-        .args(&[target1, target2])
-        .run()
-        .stdout_is_bytes(
-            [
-                b"0 0 0 'some-dir1/1'$'\\300\\n''.txt'\n".to_vec(),
-                b"0 0 0 some-dir1/2\xC0\t.txt\n".to_vec(),
-                b"0 0 0 total\n".to_vec(),
-            ]
-            .concat(),
-        );
+
+    ucmd.args(&[target1, target2]).succeeds().stdout_is_bytes(
+        [
+            b"0 0 0 'some-dir1/1'$'\\300\\n''.txt'\n".to_vec(),
+            b"0 0 0 some-dir1/2\xC0\t.txt\n".to_vec(),
+            b"0 0 0 total\n".to_vec(),
+        ]
+        .concat(),
+    );
 }
 
 #[test]
@@ -318,7 +318,7 @@ fn test_multiple_default() {
             "alice_in_wonderland.txt",
             "alice in wonderland.txt",
         ])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             "  13  109  772 lorem_ipsum.txt\n",
             "  18  204 1115 moby_dick.txt\n",
@@ -333,7 +333,7 @@ fn test_multiple_default() {
 fn test_file_empty() {
     new_ucmd!()
         .args(&["-clmwL", "emptyfile.txt"])
-        .run()
+        .succeeds()
         .stdout_is("0 0 0 0 0 emptyfile.txt\n");
 }
 
@@ -343,7 +343,7 @@ fn test_file_empty() {
 fn test_file_single_line_no_trailing_newline() {
     new_ucmd!()
         .args(&["-clmwL", "notrailingnewline.txt"])
-        .run()
+        .succeeds()
         .stdout_is("1 1 2 2 1 notrailingnewline.txt\n");
 }
 
@@ -353,7 +353,7 @@ fn test_file_single_line_no_trailing_newline() {
 fn test_file_many_empty_lines() {
     new_ucmd!()
         .args(&["-clmwL", "manyemptylines.txt"])
-        .run()
+        .succeeds()
         .stdout_is("100   0 100 100   0 manyemptylines.txt\n");
 }
 
@@ -362,7 +362,7 @@ fn test_file_many_empty_lines() {
 fn test_file_one_long_line_only_spaces() {
     new_ucmd!()
         .args(&["-clmwL", "onelongemptyline.txt"])
-        .run()
+        .succeeds()
         .stdout_is("    1     0 10001 10001 10000 onelongemptyline.txt\n");
 }
 
@@ -371,7 +371,7 @@ fn test_file_one_long_line_only_spaces() {
 fn test_file_one_long_word() {
     new_ucmd!()
         .args(&["-clmwL", "onelongword.txt"])
-        .run()
+        .succeeds()
         .stdout_is("    1     1 10001 10001 10000 onelongword.txt\n");
 }
 
@@ -402,21 +402,21 @@ fn test_file_bytes_dictate_width() {
     // five characters, filled with whitespace.
     new_ucmd!()
         .args(&["-lw", "onelongemptyline.txt"])
-        .run()
+        .succeeds()
         .stdout_is("    1     0 onelongemptyline.txt\n");
 
     // This file has zero bytes. Only one digit is required to
     // represent that.
     new_ucmd!()
         .args(&["-lw", "emptyfile.txt"])
-        .run()
+        .succeeds()
         .stdout_is("0 0 emptyfile.txt\n");
 
     // lorem_ipsum.txt contains 772 bytes, and alice_in_wonderland.txt contains
     // 302 bytes. The total is 1074 bytes, which has a width of 4
     new_ucmd!()
         .args(&["-lwc", "alice_in_wonderland.txt", "lorem_ipsum.txt"])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             "   5   57  302 alice_in_wonderland.txt\n",
             "  13  109  772 lorem_ipsum.txt\n",
@@ -425,7 +425,7 @@ fn test_file_bytes_dictate_width() {
 
     new_ucmd!()
         .args(&["-lwc", "emptyfile.txt", "."])
-        .run()
+        .fails()
         .stdout_is(STDOUT);
 }
 
@@ -449,6 +449,23 @@ fn test_read_from_directory_error() {
         .stdout_is(STDOUT);
 }
 
+#[cfg(unix)]
+#[test]
+fn test_read_error_order_with_stderr_to_stdout() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.mkdir("ioerrdir");
+
+    let expected = format!(
+        "{:>7} {:>7} {:>7} ioerrdir\nwc: ioerrdir: Is a directory\n",
+        0, 0, 0
+    );
+
+    ucmd.arg("ioerrdir")
+        .stderr_to_stdout()
+        .fails()
+        .stdout_only(expected);
+}
+
 /// Test that getting counts from nonexistent file is an error.
 #[test]
 fn test_read_from_nonexistent_file() {
@@ -466,15 +483,18 @@ fn test_files_from_pseudo_filesystem() {
     assert_ne!(result.stdout_str(), "0 /proc/cpuinfo\n");
 
     // the following block fails on Android with a "Permission denied" error
+    // also skip in case the kernel was not built with profiling support, e.g. WSL
     #[cfg(target_os = "linux")]
     {
         let (at, mut ucmd) = at_and_ucmd!();
-        let result = ucmd.arg("-c").arg("/sys/kernel/profiling").succeeds();
-        let actual = at.read("/sys/kernel/profiling").len();
-        assert_eq!(
-            result.stdout_str(),
-            format!("{actual} /sys/kernel/profiling\n")
-        );
+        if at.file_exists("/sys/kernel/profiling") {
+            let result = ucmd.arg("-c").arg("/sys/kernel/profiling").succeeds();
+            let actual = at.read("/sys/kernel/profiling").len();
+            assert_eq!(
+                result.stdout_str(),
+                format!("{actual} /sys/kernel/profiling\n")
+            );
+        }
     }
 }
 
@@ -495,8 +515,7 @@ fn test_files0_from() {
     // file
     new_ucmd!()
         .args(&["--files0-from=files0_list.txt"])
-        .run()
-        .success()
+        .succeeds()
         .stdout_is(concat!(
             "  13  109  772 lorem_ipsum.txt\n",
             "  18  204 1115 moby_dick.txt\n",
@@ -508,8 +527,7 @@ fn test_files0_from() {
     new_ucmd!()
         .args(&["--files0-from=-"])
         .pipe_in_fixture("files0_list.txt")
-        .run()
-        .success()
+        .succeeds()
         .stdout_is(concat!(
             "13 109 772 lorem_ipsum.txt\n",
             "18 204 1115 moby_dick.txt\n",
@@ -523,7 +541,7 @@ fn test_files0_from_with_stdin() {
     new_ucmd!()
         .args(&["--files0-from=-"])
         .pipe_in("lorem_ipsum.txt")
-        .run()
+        .succeeds()
         .stdout_is("13 109 772 lorem_ipsum.txt\n");
 }
 
@@ -532,7 +550,7 @@ fn test_files0_from_with_stdin_in_file() {
     new_ucmd!()
         .args(&["--files0-from=files0_list_with_stdin.txt"])
         .pipe_in_fixture("alice_in_wonderland.txt")
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             "     13     109     772 lorem_ipsum.txt\n",
             "     18     204    1115 moby_dick.txt\n",
@@ -543,7 +561,7 @@ fn test_files0_from_with_stdin_in_file() {
 
 #[test]
 fn test_files0_from_with_stdin_try_read_from_stdin() {
-    const MSG: &str = "when reading file names from stdin, no file name of '-' allowed";
+    const MSG: &str = "when reading file names from standard input, no file name of '-' allowed";
     new_ucmd!()
         .args(&["--files0-from=-"])
         .pipe_in("-")
@@ -556,16 +574,16 @@ fn test_files0_from_with_stdin_try_read_from_stdin() {
 fn test_total_auto() {
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "--total=auto"])
-        .run()
+        .succeeds()
         .stdout_is(" 13 109 772 lorem_ipsum.txt\n");
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "--tot=au"])
-        .run()
+        .succeeds()
         .stdout_is(" 13 109 772 lorem_ipsum.txt\n");
 
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=auto"])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             "  13  109  772 lorem_ipsum.txt\n",
             "  18  204 1115 moby_dick.txt\n",
@@ -577,14 +595,14 @@ fn test_total_auto() {
 fn test_total_always() {
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "--total=always"])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             " 13 109 772 lorem_ipsum.txt\n",
             " 13 109 772 total\n",
         ));
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "--total=al"])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             " 13 109 772 lorem_ipsum.txt\n",
             " 13 109 772 total\n",
@@ -592,7 +610,7 @@ fn test_total_always() {
 
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=always"])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             "  13  109  772 lorem_ipsum.txt\n",
             "  18  204 1115 moby_dick.txt\n",
@@ -604,19 +622,19 @@ fn test_total_always() {
 fn test_total_never() {
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "--total=never"])
-        .run()
+        .succeeds()
         .stdout_is(" 13 109 772 lorem_ipsum.txt\n");
 
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=never"])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             "  13  109  772 lorem_ipsum.txt\n",
             "  18  204 1115 moby_dick.txt\n",
         ));
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=n"])
-        .run()
+        .succeeds()
         .stdout_is(concat!(
             "  13  109  772 lorem_ipsum.txt\n",
             "  18  204 1115 moby_dick.txt\n",
@@ -627,16 +645,16 @@ fn test_total_never() {
 fn test_total_only() {
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "--total=only"])
-        .run()
+        .succeeds()
         .stdout_is("13 109 772\n");
 
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=only"])
-        .run()
+        .succeeds()
         .stdout_is("31 313 1887\n");
     new_ucmd!()
         .args(&["lorem_ipsum.txt", "moby_dick.txt", "--t=o"])
-        .run()
+        .succeeds()
         .stdout_is("31 313 1887\n");
 }
 
@@ -650,8 +668,7 @@ fn test_zero_length_files() {
         new_ucmd!()
             .args(&["--files0-from=-"])
             .pipe_in(&LIST[..l])
-            .run()
-            .failure()
+            .fails()
             .stdout_is(concat!(
                 "18 204 1115 moby_dick.txt\n",
                 "5 57 302 alice_in_wonderland.txt\n",
@@ -675,8 +692,7 @@ fn test_zero_length_files() {
                 .copied()
                 .collect::<Vec<_>>(),
         )
-        .run()
-        .failure()
+        .fails()
         .stdout_is(concat!(
             "18 204 1115 moby_dick.txt\n",
             "5 57 302 alice_in_wonderland.txt\n",
@@ -695,8 +711,7 @@ fn test_zero_length_files() {
 fn test_files0_errors_quoting() {
     new_ucmd!()
         .args(&["--files0-from=files0 with nonexistent.txt"])
-        .run()
-        .failure()
+        .fails()
         .stderr_is(concat!(
             "wc: this_file_does_not_exist.txt: No such file or directory\n",
             "wc: 'files0 with nonexistent.txt':2: invalid zero-length file name\n",
@@ -793,11 +808,116 @@ fn files0_from_dir() {
 fn test_args_override() {
     new_ucmd!()
         .args(&["-ll", "-l", "alice_in_wonderland.txt"])
-        .run()
+        .succeeds()
         .stdout_is("5 alice_in_wonderland.txt\n");
 
     new_ucmd!()
         .args(&["--total=always", "--total=never", "alice_in_wonderland.txt"])
-        .run()
+        .succeeds()
         .stdout_is("  5  57 302 alice_in_wonderland.txt\n");
 }
+
+#[test]
+fn wc_w_words_with_emoji_separator() {
+    new_ucmd!()
+        .args(&["-w"])
+        .pipe_in("foo 💐 bar\n")
+        .succeeds()
+        .stdout_contains("3");
+}
+
+#[test]
+fn test_invalid_byte_sequence_word_count() {
+    // wc should count invalid byte sequences as words
+    // Input: "a \xff b\n" should produce: 1 line, 3 words, 6 bytes
+    new_ucmd!()
+        .pipe_in([b'a', b' ', 0xff, b' ', b'b', b'\n'])
+        .succeeds()
+        .stdout_is("      1       3       6\n");
+}
+
+#[cfg(unix)]
+#[test]
+fn test_simd_respects_glibc_tunables() {
+    // Ensure debug output reflects that SIMD paths are disabled via GLIBC_TUNABLES
+    let debug_output = new_ucmd!()
+        .args(&["-l", "--debug", "/dev/null"])
+        .env("GLIBC_TUNABLES", "glibc.cpu.hwcaps=-AVX2,-AVX512F")
+        .succeeds()
+        .stderr_str()
+        .to_string();
+    assert!(
+        !debug_output.contains("using hardware support"),
+        "SIMD should be reported as disabled when GLIBC_TUNABLES blocks AVX features: {debug_output}"
+    );
+    assert!(
+        debug_output.contains("hardware support disabled"),
+        "Debug output should acknowledge GLIBC_TUNABLES restrictions: {debug_output}"
+    );
+
+    // WC results should be identical with and without GLIBC_TUNABLES overrides
+    let sample_sizes = [0usize, 1, 7, 128, 513, 999];
+    use std::fmt::Write as _;
+    for &lines in &sample_sizes {
+        let content: String = (0..lines).fold(String::new(), |mut acc, i| {
+            // Build the input buffer efficiently without allocating per line.
+            let _ = writeln!(acc, "{i}");
+            acc
+        });
+
+        let base = new_ucmd!()
+            .arg("-l")
+            .pipe_in(content.clone())
+            .succeeds()
+            .stdout_str()
+            .trim()
+            .to_string();
+
+        let no_avx512 = new_ucmd!()
+            .arg("-l")
+            .env("GLIBC_TUNABLES", "glibc.cpu.hwcaps=-AVX512F")
+            .pipe_in(content.clone())
+            .succeeds()
+            .stdout_str()
+            .trim()
+            .to_string();
+
+        let no_avx2_avx512 = new_ucmd!()
+            .arg("-l")
+            .env("GLIBC_TUNABLES", "glibc.cpu.hwcaps=-AVX2,-AVX512F")
+            .pipe_in(content)
+            .succeeds()
+            .stdout_str()
+            .trim()
+            .to_string();
+
+        assert_eq!(
+            base, no_avx512,
+            "Line counts should not change when AVX512 is disabled (lines={lines})"
+        );
+        assert_eq!(
+            base, no_avx2_avx512,
+            "Line counts should not change when AVX2/AVX512 are disabled (lines={lines})"
+        );
+    }
+}
+
+#[test]
+fn test_posixly_correct_whitespace() {
+    let input = "word\u{00A0}word"; // Non-breaking space
+
+    // Default: Unicode whitespace is respected
+    new_ucmd!()
+        .arg("-w")
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is("2\n");
+
+    // POSIXLY_CORRECT: Only ASCII whitespace
+    new_ucmd!()
+        .arg("-w")
+        .env("POSIXLY_CORRECT", "1")
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is("1\n");
+}
